from ast import literal_eval

import cmd2
from rich.console import Console
from rich.table import Table

from moonshot.api import api_delete_prompt_template, api_get_all_prompt_template_detail
from moonshot.integrations.cli.cli_errors import (
    ERROR_COMMON_DELETE_PROMPT_TEMPLATE_PROMPT_TEMPLATE_VALIDATION,
    ERROR_COMMON_LIST_CONNECTOR_TYPES_PAGINATION_VALIDATION,
    ERROR_COMMON_LIST_CONNECTOR_TYPES_PAGINATION_VALIDATION_1,
    ERROR_COMMON_LIST_PROMPT_TEMPLATES_FIND_VALIDATION,
)
from moonshot.integrations.cli.utils.process_data import filter_data

console = Console()


# ------------------------------------------------------------------------------
# CLI Functions
# ------------------------------------------------------------------------------
def list_prompt_templates(args) -> list | None:
    """
    List all prompt templates available.

    Args:
        args: A namespace object from argparse. It should have an optional attribute:
        find (str): Optional field to find prompt template(s) with a keyword.
        pagination (str): Optional field to paginate prompt templates.

    Returns:
        list | None: A list of PromptTemplate or None if there is no result.
    """
    try:
        if args.find is not None:
            if not isinstance(args.find, str) or not args.find:
                raise TypeError(ERROR_COMMON_LIST_PROMPT_TEMPLATES_FIND_VALIDATION)

        if args.pagination is not None:
            if not isinstance(args.pagination, str) or not args.pagination:
                raise TypeError(ERROR_COMMON_LIST_CONNECTOR_TYPES_PAGINATION_VALIDATION)
            try:
                pagination = literal_eval(args.pagination)
                if not (
                    isinstance(pagination, tuple)
                    and len(pagination) == 2
                    and all(isinstance(i, int) for i in pagination)
                ):
                    raise ValueError(
                        ERROR_COMMON_LIST_CONNECTOR_TYPES_PAGINATION_VALIDATION_1
                    )
            except (ValueError, SyntaxError):
                raise ValueError(
                    ERROR_COMMON_LIST_CONNECTOR_TYPES_PAGINATION_VALIDATION_1
                )
        else:
            pagination = ()

        prompt_templates_list = api_get_all_prompt_template_detail()
        keyword = args.find.lower() if args.find else ""

        if prompt_templates_list:
            filtered_prompt_templates_list = filter_data(
                prompt_templates_list, keyword, pagination
            )
            if filtered_prompt_templates_list:
                _display_prompt_templates(filtered_prompt_templates_list)
                return filtered_prompt_templates_list

        console.print("[red]There are no prompt templates found.[/red]")
        return None
    except Exception as e:
        print(f"[list_prompt_templates]: {str(e)}")


def delete_prompt_template(args) -> None:
    """
    Deletes a prompt_template after confirming with the user.

    Args:
        args (object): The arguments object. It should have a 'prompt_template' attribute
                       which is the ID of the prompt template to delete.
    """
    # Confirm with the user before deleting a prompt template
    confirmation = console.input(
        "[bold red]Are you sure you want to delete the prompt template (y/N)? [/]"
    )
    if confirmation.lower() != "y":
        console.print("[bold yellow]Prompt template deletion cancelled.[/]")
        return
    try:
        if (
            args.prompt_template is None
            or not isinstance(args.prompt_template, str)
            or not args.prompt_template
        ):
            raise ValueError(
                ERROR_COMMON_DELETE_PROMPT_TEMPLATE_PROMPT_TEMPLATE_VALIDATION
            )
        api_delete_prompt_template(args.prompt_template)
        print("[delete_prompt_template]: Prompt template deleted.")
    except Exception as e:
        print(f"[delete_prompt_template]: {str(e)}")


# ------------------------------------------------------------------------------
# Helper functions: Display on cli
# ------------------------------------------------------------------------------
def _display_prompt_templates(prompt_templates) -> None:
    """
    Display the list of prompt templates in a formatted table.

    This function takes a list of prompt templates and displays them in a formatted table.
    Each row in the table represents a prompt template with its ID, name, description, and contents.
    If the list of prompt templates is empty, it prints a message indicating that no prompt templates were found.

    Args:
        prompt_templates (list): A list of dictionaries, each representing a prompt template.
    """
    table = Table(
        title="List of Prompt Templates",
        show_lines=True,
        expand=True,
        header_style="bold",
    )
    table.add_column("No.", width=2)
    table.add_column("Prompt Template", justify="left", width=50)
    table.add_column("Contains", justify="left", width=48, overflow="fold")
    for idx, prompt_template in enumerate(prompt_templates, 1):
        (id, name, description, contents, *other_args) = prompt_template.values()
        idx = prompt_template.get("idx", idx)
        prompt_info = f"[red]id: {id}[/red]\n\n[blue]{name}[/blue]\n{description}"
        table.add_section()
        table.add_row(str(idx), prompt_info, contents)
    console.print(table)


# Delete prompt template arguments
delete_prompt_template_args = cmd2.Cmd2ArgumentParser(
    description="Delete a prompt template.",
    epilog="Example:\n delete_prompt_template squad-shifts",
)

delete_prompt_template_args.add_argument(
    "prompt_template", type=str, help="The ID of the prompt template to delete"
)

# List prompt template arguments
list_prompt_templates_args = cmd2.Cmd2ArgumentParser(
    description="List all prompt templates.",
    epilog='Example:\n list_prompt_templates -f "toxicity"',
)

list_prompt_templates_args.add_argument(
    "-f",
    "--find",
    type=str,
    help="Optional field to find prompt template(s) with keyword",
    nargs="?",
)

list_prompt_templates_args.add_argument(
    "-p",
    "--pagination",
    type=str,
    help="Optional tuple to paginate prompt template(s). E.g. (2,10) returns 2nd page with 10 items in each page.",
    nargs="?",
)



from __future__ import annotations

from pathlib import Path

from pydantic import validate_call

from moonshot.src.configs.env_variables import EnvVariables
from moonshot.src.storage.storage import Storage
from moonshot.src.utils.log import configure_logger

# Create a logger for this module
logger = configure_logger(__name__)


class Result:
    @staticmethod
    @validate_call
    def read(result_id: str) -> dict:
        """
        Reads the result data from storage for a given result ID.

        This method attempts to retrieve the result data associated with the specified result ID from storage.
        If the data is found, it is returned as a dictionary. If no data is found, an exception is raised and
        an error message is printed.

        Args:
            result_id (str): The unique identifier of the result to be read.

        Returns:
            dict: A dictionary containing the result data if found.

        Raises:
            Exception: If no result data is found or if an error occurs during the read operation.
        """
        try:
            if result_id:
                return Result._read_result(result_id)
            else:
                raise RuntimeError("Result ID is empty")

        except Exception as e:
            logger.error(f"Failed to read result: {str(e)}")
            raise e

    @staticmethod
    def _read_result(result_id: str) -> dict:
        """
        Reads the result data from storage for a given result ID.

        This method attempts to retrieve the result data associated with the specified result ID from storage.
        If the data is found, it is returned as a dictionary. If no data is found, a RuntimeError is raised.

        Args:
            result_id (str): The unique identifier of the result to be read.

        Returns:
            dict: A dictionary containing the result data.

        Raises:
            RuntimeError: If no result data is found for the given result ID.
        """
        obj_results = Storage.read_object(EnvVariables.RESULTS.name, result_id, "json")
        if obj_results:
            return obj_results
        else:
            raise RuntimeError(f"Unable to get results for {result_id}.")

    @staticmethod
    @validate_call
    def delete(result_id: str) -> bool:
        """
        Deletes the result data associated with the given result ID from storage.

        This method attempts to delete the result data identified by the specified result ID from storage.
        If the deletion is successful, it returns True. If an exception occurs during the deletion process,
        an error message is printed and the exception is re-raised.

        Args:
            result_id (str): The unique identifier of the result to be deleted.

        Returns:
            bool: True if the result data was successfully deleted.

        Raises:
            Exception: If an error occurs during the deletion process.
        """
        try:
            Storage.delete_object(EnvVariables.RESULTS.name, result_id, "json")
            return True

        except Exception as e:
            logger.error(f"Failed to delete result: {str(e)}")
            raise e

    @staticmethod
    def get_available_items() -> tuple[list[str], list[dict]]:
        """
        Retrieves the list of available result IDs and their corresponding result data.

        This method queries the storage to obtain all the result objects, filters out any that are not relevant
        (e.g., internal use objects with "__" in their name), and then reads the result data for each remaining
        result ID. It returns a tuple containing a list of result IDs and a list of dictionaries with the result
        data.

        Returns:
            tuple[list[str], list[dict]]: A tuple with the first element being a list of result IDs and the
            second element being a list of dictionaries containing the result data for each ID.
        """
        try:
            retn_results = []
            retn_results_ids = []

            for result in Storage.get_objects(EnvVariables.RESULTS.name, "json"):
                if "__" in result:
                    continue
                result_info = Result._read_result(Path(result).stem)
                retn_results.append(result_info)
                retn_results_ids.append(Path(result).stem)
            return retn_results_ids, retn_results

        except Exception as e:
            logger.error(f"Failed to get available results: {str(e)}")
            raise e



from typing import Optional

from dependency_injector.wiring import Provide, inject
from fastapi import APIRouter, Depends, HTTPException, Query

from ..container import Container
from ..schemas.recipe_create_dto import RecipeCreateDTO, RecipeUpdateDTO
from ..schemas.recipe_response_model import RecipeResponseModel
from ..services.recipe_service import RecipeService
from ..services.utils.exceptions_handler import ServiceException

router = APIRouter(tags=["Recipe"])


@router.post("/api/v1/recipes")
@inject
def create_recipe(
    recipe_data: RecipeCreateDTO,
    recipe_service: RecipeService = Depends(Provide[Container.recipe_service]),
) -> dict[str, str]:
    """
    Endpoint to add a new recipe to the database.

    Parameters:
        recipe_data (RecipeCreateDTO): The data transfer object containing the recipe details.
        recipe_service (RecipeService): The service layer responsible for the creation logic.

    Returns:
        dict[str, str]: A dictionary with a message indicating the successful creation of the recipe.

    Raises:
        HTTPException: 404 if the recipe cannot be created because the file is not found.
                       400 if there is a validation error with the provided data.
                       500 for any other internal server error.
    """
    try:
        recipe_service.create_recipe(recipe_data)
        return {"message": "Recipe created successfully"}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to create recipe: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to create recipe: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to create recipe: {e.msg}"
            )


@router.get("/api/v1/recipes")
@inject
def get_all_recipes(
    ids: Optional[str] = Query(None, description="Get recipes to query"),
    tags: Optional[str] = Query(None, description="Filter recipes by tags"),
    categories: str = Query(None, description="Filter recipes by categories"),
    sort_by: Optional[str] = Query(
        None, description="Sort recipes by a specific field"
    ),
    count: bool = Query(False, description="Whether to include the count of recipes"),
    recipe_service: RecipeService = Depends(Provide[Container.recipe_service]),
) -> list[RecipeResponseModel]:
    """
    Endpoint to retrieve all recipes from the database, with optional filters, sorting, and count inclusion.

    Parameters:
        ids (str, optional): Filter to retrieve recipes by list of comma separated recipe ids.
        tags (str, optional): Filter to retrieve recipes by tags.
        categories (str, optional): Filter to retrieve recipes by categories.
        sort_by (str, optional): Parameter to sort recipes by a specific field.
        count (bool, optional): Flag to indicate whether to include the count of recipes in the response.
        recipe_service (RecipeService): The service layer responsible for the retrieval logic.

    Returns:
        list[RecipeResponseModel]: A list of recipe, filtered, sorted, and with counts.

    Raises:
        HTTPException: 404 if no recipes are found.
                       400 if there is a validation error with the provided data.
                       500 for any other internal server error.
    """
    try:
        recipes = recipe_service.get_all_recipes(
            tags=tags, categories=categories, sort_by=sort_by, count=count, ids=ids
        )
        return recipes
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve recipes: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve recipes: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve recipes: {e.msg}"
            )


@router.get("/api/v1/recipes/name")
@inject
def get_all_recipes_name(
    recipe_service: RecipeService = Depends(Provide[Container.recipe_service]),
) -> list[str]:
    """
    Endpoint to retrieve all recipe names from the database.

    Parameters:
        recipe_service (RecipeService): The service layer responsible for retrieving recipe names.

    Returns:
        list[str]: A list of recipe names.

    Raises:
        HTTPException: 404 if no recipe names are found.
                       400 if there is a validation error with the provided data.
                       500 for any other internal server error.
    """
    try:
        recipes = recipe_service.get_all_recipes_name()
        return recipes
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve recipe names: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve recipe names: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve recipe names: {e.msg}"
            )


@router.put("/api/v1/recipes/{recipe_id}")
@inject
async def update_recipe(
    recipe_data: RecipeUpdateDTO,
    recipe_id: str,
    recipe_service: RecipeService = Depends(Provide[Container.recipe_service]),
) -> dict[str, str]:
    """
    Endpoint to update an existing recipe in the database by its ID.

    Parameters:
        recipe_data (RecipeCreateDTO): The data transfer object containing the updated recipe details.
        recipe_id (str): The unique identifier of the recipe to update.
        recipe_service (RecipeService): The service layer responsible for the update logic.

    Returns:
        dict[str, str]: A dictionary with a message indicating the successful update of the recipe.

    Raises:
        HTTPException: 404 if the recipe to be updated cannot be found.
                       400 if there is a validation error with the provided data.
                       500 for any other internal server error.
    """
    try:
        recipe_service.update_recipe(recipe_data, recipe_id)
        return {"message": "Recipe updated successfully"}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to update recipe: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to update recipe: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to update recipe: {e.msg}"
            )


@router.delete("/api/v1/recipes/{recipe_id}")
@inject
def delete_recipe(
    recipe_id: str,
    recipe_service: RecipeService = Depends(Provide[Container.recipe_service]),
) -> dict[str, str]:
    """
    Endpoint to delete a recipe from the database by its ID.

    Parameters:
        recipe_id (str): The unique identifier of the recipe to delete.
        recipe_service (RecipeService): The service layer responsible for the deletion logic.

    Returns:
        dict[str, str]: A dictionary with a message indicating the successful deletion of the recipe.

    Raises:
        HTTPException: 404 if the recipe to be deleted cannot be found.
                       400 if there is a validation error with the provided data.
                       500 for any other internal server error.
    """
    try:
        recipe_service.delete_recipe(recipe_id)
        return {"message": "Recipe deleted successfully"}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to delete recipe: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to delete recipe: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to delete recipe: {e.msg}"
            )



import os
import tempfile

from dependency_injector.wiring import Provide, inject
from fastapi import APIRouter, Depends, File, Form, HTTPException, UploadFile

from ..container import Container
from ..schemas.dataset_create_dto import CSV_Dataset_DTO, HF_Dataset_DTO
from ..schemas.dataset_response_dto import DatasetResponseDTO
from ..services.dataset_service import DatasetService
from ..services.utils.exceptions_handler import ServiceException

router = APIRouter(tags=["Datasets"])


@router.post("/api/v1/datasets/file")
@inject
async def upload_dataset(
    file: UploadFile = File(...),
    name: str = Form(..., min_length=1),
    description: str = Form(default="", min_length=1),
    license: str = Form(default=""),
    reference: str = Form(default=""),
    dataset_service: DatasetService = Depends(Provide[Container.dataset_service]),
) -> str:
    """
    Convert a CSV dataset to the desired format.

    Args:
        dataset_data (CSV_Dataset_DTO): The data required to convert the dataset.
        dataset_service (DatasetService, optional): The service responsible for converting the dataset.
        Defaults to Depends(Provide[Container.dataset_service]).

    Returns:
        str: The path to the newly created dataset.

    Raises:
        HTTPException: An error with status code 404 if the dataset file is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """

    # Create a temporary file with a secure random name
    with tempfile.NamedTemporaryFile(
        delete=False, suffix=os.path.splitext(file.filename)[1]
    ) as tmp_file:
        content = await file.read()
        tmp_file.write(content)
        temp_file_path = tmp_file.name

    try:
        # Create the DTO with the form data including optional fields
        dataset_data = CSV_Dataset_DTO(
            name=name,
            description=description,
            license=license,
            reference=reference,
            file_path=temp_file_path,
        )
        return dataset_service.convert_dataset(dataset_data)
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to convert dataset: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to convert dataset: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to convert dataset: {e.msg}"
            )
    finally:
        # Clean up the temporary file
        if os.path.exists(temp_file_path):
            os.unlink(temp_file_path)


@router.post("/api/v1/datasets/hf")
@inject
def download_dataset(
    dataset_data: HF_Dataset_DTO,
    dataset_service: DatasetService = Depends(Provide[Container.dataset_service]),
) -> str:
    """
    Download a dataset from Hugging Face using the provided dataset data.

    Args:
        dataset_data (HF_Dataset_DTO): The data required to download the dataset.
        dataset_service (DatasetService, optional): The service responsible for downloading the dataset.
        Defaults to Depends(Provide[Container.dataset_service]).

    Returns:
        str: The path to the newly downloaded dataset.

    Raises:
        HTTPException: An error with status code 404 if the dataset file is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return dataset_service.download_dataset(dataset_data)
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to download dataset: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to download dataset: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to download dataset: {e.msg}"
            )


@router.get("/api/v1/datasets")
@inject
def get_all_datasets(
    dataset_service: DatasetService = Depends(Provide[Container.dataset_service]),
) -> list[DatasetResponseDTO]:
    """
    Retrieve all datasets from the database.

    Args:
        dataset_service (DatasetService): The service responsible for retrieving datasets.

    Returns:
        list: A list of all datasets.

    Raises:
        HTTPException: An error with status code 404 if no datasets are found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return dataset_service.get_all_datasets()
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve datasets: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve datasets: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve datasets: {e.msg}"
            )


@router.get("/api/v1/datasets/name")
@inject
def get_all_datasets_name(
    dataset_service: DatasetService = Depends(Provide[Container.dataset_service]),
) -> list[str]:
    """
    Retrieve the names of all datasets from the database.

    Args:
        dataset_service (DatasetService): The service responsible for retrieving dataset names.

    Returns:
        list[str]: A list of dataset names.

    Raises:
        HTTPException: An error with status code 404 if no dataset names are found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return dataset_service.get_all_datasets_name()
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve dataset names: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve dataset names: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve dataset names: {e.msg}"
            )


@router.delete("/api/v1/datasets/{dataset_id}")
@inject
def delete_dataset(
    dataset_id: str,
    dataset_service: DatasetService = Depends(Provide[Container.dataset_service]),
) -> dict[str, str]:
    """
    Delete a dataset from the database by its ID.

    Args:
        dataset_id (str): The unique identifier of the dataset to delete.
        dataset_service (DatasetService): The service responsible for deleting the dataset.

    Returns:
        dict[str, str]: A message indicating the successful deletion of the dataset.

    Raises:
        HTTPException: An error with status code 404 if the dataset is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        dataset_service.delete_dataset(dataset_id)
        return {"message": "Dataset deleted successfully"}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to delete dataset: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to delete dataset: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to delete dataset: {e.msg}"
            )



import argparse

import cmd2

from moonshot.integrations.cli.redteam.attack_module import (
    delete_attack_module,
    delete_attack_module_args,
    list_attack_modules,
    list_attack_modules_args,
)
from moonshot.integrations.cli.redteam.context_strategy import (
    clear_context_strategy,
    delete_context_strategy,
    delete_context_strategy_args,
    list_context_strategies,
    list_context_strategies_args,
    use_context_strategy,
    use_context_strategy_args,
)
from moonshot.integrations.cli.redteam.prompt_template import (
    clear_prompt_template,
    use_prompt_template,
    use_prompt_template_args,
)
from moonshot.integrations.cli.redteam.session import (
    add_bookmark,
    add_bookmark_args,
    automated_rt_session_args,
    delete_bookmark,
    delete_bookmark_args,
    delete_session,
    delete_session_args,
    end_session,
    export_bookmarks,
    export_bookmarks_args,
    list_bookmarks,
    list_bookmarks_args,
    list_sessions,
    list_sessions_args,
    new_session,
    new_session_args,
    run_attack_module,
    show_prompts,
    use_bookmark,
    use_bookmark_args,
    use_session,
    use_session_args,
    view_bookmark,
    view_bookmark_args,
)


@cmd2.with_default_category("Moonshot RedTeaming")
class RedTeamCommandSet(cmd2.CommandSet):
    def __init__(self):
        super().__init__()

    @cmd2.with_argparser(new_session_args)
    def do_new_session(self, args: argparse.Namespace) -> None:
        new_session(args)

    def do_end_session(self, _: cmd2.Statement) -> None:
        end_session()

    @cmd2.with_argparser(list_sessions_args)
    def do_list_sessions(self, args: argparse.Namespace) -> None:
        list_sessions(args)

    @cmd2.with_argparser(use_session_args)
    def do_use_session(self, args: argparse.Namespace) -> None:
        use_session(args)

    @cmd2.with_argparser(use_prompt_template_args)
    def do_use_prompt_template(self, args: argparse.Namespace) -> None:
        use_prompt_template(args)

    def do_clear_prompt_template(self, _: cmd2.Statement) -> None:
        clear_prompt_template()

    @cmd2.with_argparser(list_context_strategies_args)
    def do_list_context_strategies(self, args: argparse.Namespace) -> None:
        list_context_strategies(args)

    @cmd2.with_argparser(use_context_strategy_args)
    def do_use_context_strategy(self, args: argparse.Namespace) -> None:
        use_context_strategy(args)

    def do_clear_context_strategy(self, _: cmd2.Statement) -> None:
        clear_context_strategy()

    @cmd2.with_argparser(automated_rt_session_args)
    def do_run_attack_module(self, args: argparse.Namespace) -> None:
        run_attack_module(args)

    @cmd2.with_argparser(list_attack_modules_args)
    def do_list_attack_modules(self, args: argparse.Namespace) -> None:
        list_attack_modules(args)

    @cmd2.with_argparser(delete_session_args)
    def do_delete_session(self, args: argparse.Namespace) -> None:
        delete_session(args)

    @cmd2.with_argparser(delete_context_strategy_args)
    def do_delete_context_strategy(self, args: argparse.Namespace) -> None:
        delete_context_strategy(args)

    @cmd2.with_argparser(delete_attack_module_args)
    def do_delete_attack_module(self, args: argparse.Namespace) -> None:
        delete_attack_module(args)

    @cmd2.with_argparser(add_bookmark_args)
    def do_add_bookmark(self, args: argparse.Namespace) -> None:
        add_bookmark(args)

    @cmd2.with_argparser(use_bookmark_args)
    def do_use_bookmark(self, args: argparse.Namespace) -> None:
        use_bookmark(args)

    @cmd2.with_argparser(delete_bookmark_args)
    def do_delete_bookmark(self, args: argparse.Namespace) -> None:
        delete_bookmark(args)

    @cmd2.with_argparser(list_bookmarks_args)
    def do_list_bookmarks(self, args: argparse.Namespace) -> None:
        list_bookmarks(args)

    @cmd2.with_argparser(view_bookmark_args)
    def do_view_bookmark(self, args: argparse.Namespace) -> None:
        view_bookmark(args)

    @cmd2.with_argparser(export_bookmarks_args)
    def do_export_bookmarks(self, args: argparse.Namespace) -> None:
        export_bookmarks(args)

    def do_show_prompts(self, _: cmd2.Statement) -> None:
        show_prompts()



import argparse

import cmd2

from moonshot.integrations.cli.benchmark.cookbook import (
    add_cookbook,
    add_cookbook_args,
    delete_cookbook,
    delete_cookbook_args,
    list_cookbooks,
    list_cookbooks_args,
    run_cookbook,
    run_cookbook_args,
    update_cookbook,
    update_cookbook_args,
    view_cookbook,
    view_cookbook_args,
)
from moonshot.integrations.cli.benchmark.datasets import (
    delete_dataset,
    delete_dataset_args,
    list_datasets,
    list_datasets_args,
    view_dataset,
    view_dataset_args,
)
from moonshot.integrations.cli.benchmark.metrics import (
    delete_metric,
    delete_metric_args,
    list_metrics,
    list_metrics_args,
    view_metric,
    view_metric_args,
)
from moonshot.integrations.cli.benchmark.recipe import (
    add_recipe,
    add_recipe_args,
    delete_recipe,
    delete_recipe_args,
    list_recipes,
    list_recipes_args,
    run_recipe,
    run_recipe_args,
    update_recipe,
    update_recipe_args,
    view_recipe,
    view_recipe_args,
)
from moonshot.integrations.cli.benchmark.result import (
    delete_result,
    delete_result_args,
    list_results,
    list_results_args,
    view_result,
    view_result_args,
)
from moonshot.integrations.cli.benchmark.run import (
    list_runs,
    list_runs_args,
    view_run,
    view_run_args,
)
from moonshot.integrations.cli.benchmark.runner import (
    delete_runner,
    delete_runner_args,
    list_runners,
    view_runner,
    view_runner_args,
)


@cmd2.with_default_category("Moonshot Benchmarking")
class BenchmarkCommandSet(cmd2.CommandSet):
    def __init__(self):
        super().__init__()

    # ------------------------------------------------------------------------------
    # List contents
    # ------------------------------------------------------------------------------

    @cmd2.with_argparser(list_cookbooks_args)
    def do_list_cookbooks(self, args: argparse.Namespace) -> None:
        list_cookbooks(args)

    @cmd2.with_argparser(list_recipes_args)
    def do_list_recipes(self, args: argparse.Namespace) -> None:
        list_recipes(args)

    @cmd2.with_argparser(list_results_args)
    def do_list_results(self, args: argparse.Namespace) -> None:
        list_results(args)

    def do_list_runners(self, _: cmd2.Statement) -> None:
        list_runners()

    @cmd2.with_argparser(list_runs_args)
    def do_list_runs(self, args: argparse.Namespace) -> None:
        list_runs(args)

    @cmd2.with_argparser(list_metrics_args)
    def do_list_metrics(self, args: argparse.Namespace) -> None:
        list_metrics(args)

    @cmd2.with_argparser(list_datasets_args)
    def do_list_datasets(self, args: argparse.Namespace) -> None:
        list_datasets(args)

    # ------------------------------------------------------------------------------
    # Add contents
    # ------------------------------------------------------------------------------

    @cmd2.with_argparser(add_cookbook_args)
    def do_add_cookbook(self, args: argparse.Namespace) -> None:
        add_cookbook(args)

    @cmd2.with_argparser(add_recipe_args)
    def do_add_recipe(self, args: argparse.Namespace) -> None:
        add_recipe(args)

    # ------------------------------------------------------------------------------
    # Delete contents
    # ------------------------------------------------------------------------------

    @cmd2.with_argparser(delete_cookbook_args)
    def do_delete_cookbook(self, args: argparse.Namespace) -> None:
        delete_cookbook(args)

    @cmd2.with_argparser(delete_recipe_args)
    def do_delete_recipe(self, args: argparse.Namespace) -> None:
        delete_recipe(args)

    @cmd2.with_argparser(delete_result_args)
    def do_delete_result(self, args: argparse.Namespace) -> None:
        delete_result(args)

    @cmd2.with_argparser(delete_runner_args)
    def do_delete_runner(self, args: argparse.Namespace) -> None:
        delete_runner(args)

    @cmd2.with_argparser(delete_metric_args)
    def do_delete_metric(self, args: argparse.Namespace) -> None:
        delete_metric(args)

    @cmd2.with_argparser(delete_dataset_args)
    def do_delete_dataset(self, args: argparse.Namespace) -> None:
        delete_dataset(args)

    # ------------------------------------------------------------------------------
    # Update contents
    # ------------------------------------------------------------------------------

    @cmd2.with_argparser(update_cookbook_args)
    def do_update_cookbook(self, args: argparse.Namespace) -> None:
        update_cookbook(args)

    @cmd2.with_argparser(update_recipe_args)
    def do_update_recipe(self, args: argparse.Namespace) -> None:
        update_recipe(args)

    # ------------------------------------------------------------------------------
    # Run contents
    # ------------------------------------------------------------------------------

    @cmd2.with_argparser(run_cookbook_args)
    def do_run_cookbook(self, args: argparse.Namespace) -> None:
        run_cookbook(args)

    @cmd2.with_argparser(run_recipe_args)
    def do_run_recipe(self, args: argparse.Namespace) -> None:
        run_recipe(args)

    # ------------------------------------------------------------------------------
    # View contents
    # ------------------------------------------------------------------------------

    @cmd2.with_argparser(view_cookbook_args)
    def do_view_cookbook(self, args: argparse.Namespace) -> None:
        view_cookbook(args)

    @cmd2.with_argparser(view_recipe_args)
    def do_view_recipe(self, args: argparse.Namespace) -> None:
        view_recipe(args)

    @cmd2.with_argparser(view_result_args)
    def do_view_result(self, args: argparse.Namespace) -> None:
        view_result(args)

    @cmd2.with_argparser(view_runner_args)
    def do_view_runner(self, args: argparse.Namespace) -> None:
        view_runner(args)

    @cmd2.with_argparser(view_run_args)
    def do_view_run(self, args: argparse.Namespace) -> None:
        view_run(args)

    @cmd2.with_argparser(view_metric_args)
    def do_view_metric(self, args: argparse.Namespace) -> None:
        view_metric(args)

    @cmd2.with_argparser(view_dataset_args)
    def do_view_dataset(self, args: argparse.Namespace) -> None:
        view_dataset(args)



from dependency_injector.wiring import Provide, inject
from fastapi import APIRouter, Depends, HTTPException

from ..container import Container
from ..services.benchmark_result_service import BenchmarkResultService
from ..services.utils.exceptions_handler import ServiceException

router = APIRouter(tags=["Benchmark Results"])


@router.get("/api/v1/benchmarks/results")
@inject
async def get_all_results(
    benchmark_result_service: BenchmarkResultService = Depends(
        Provide[Container.benchmark_result_service]
    ),
) -> list[dict]:
    """
    Retrieve all benchmark results.

    This endpoint retrieves a list of all benchmark results from the database. Each benchmark result is
    represented as a dictionary containing its associated data.

    Args:
        benchmark_result_service (BenchmarkResultService): The service responsible for fetching benchmark results.

    Returns:
        list[dict]: A list of dictionaries, each representing a single benchmark result.

    Raises:
        HTTPException: Raised if the results file cannot be found (404) or if an unspecified error occurs (500).
    """
    try:
        results = benchmark_result_service.get_all_results()
        return results
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve results: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve results: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve results: {e.msg}"
            )


@router.get("/api/v1/benchmarks/results/name")
@inject
async def get_all_results_name(
    benchmark_result_service: BenchmarkResultService = Depends(
        Provide[Container.benchmark_result_service]
    ),
):
    """
    Get all benchmark result names from the database.

    This endpoint retrieves the names of all benchmark results stored in the database.

    Args:
        benchmark_result_service (BenchmarkResultService): The service responsible for fetching
        the names of the benchmark results.

    Returns:
        A list of all benchmark result names.

    Raises:
        HTTPException: An error occurred while trying to find the result names file (404),
                       a validation error occurred (400), or
                       an unspecified error occurred (500).
    """
    try:
        results = benchmark_result_service.get_all_result_name()
        return results
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve result name: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve result name: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve result name: {e.msg}"
            )


@router.get("/api/v1/benchmarks/results/{result_id}")
@inject
async def get_one_results(
    result_id: str,
    benchmark_result_service: BenchmarkResultService = Depends(
        Provide[Container.benchmark_result_service]
    ),
):
    """
    Retrieve a single benchmark result by its ID.

    This endpoint fetches the details of a specific benchmark result identified by the provided result_id.

    Args:
        result_id (str): The unique identifier of the benchmark result to retrieve.
        benchmark_result_service (BenchmarkResultService): The service responsible for fetching the benchmark result.

    Returns:
        dict: A dictionary containing the details of the benchmark result.

    Raises:
        HTTPException: An error occurred while trying to find the results file (404) or
                       an unspecified error occurred (500).
    """
    try:
        results = benchmark_result_service.get_result_by_id(result_id)
        return results
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve result: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve result: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve result: {e.msg}"
            )


@router.delete("/api/v1/benchmarks/results/{result_id}")
@inject
def delete_result(
    result_id: str,
    benchmark_result_service: BenchmarkResultService = Depends(
        Provide[Container.benchmark_result_service]
    ),
) -> dict[str, str] | tuple[dict[str, str], int]:
    """
    Delete a benchmark result by its ID.

    This endpoint deletes a specific benchmark result identified by the provided result_id.

    Args:
        result_id (str): The unique identifier of the benchmark result to delete.
        benchmark_result_service (BenchmarkResultService): The service responsible for deleting the benchmark result.

    Returns:
        dict[str, str] | tuple[dict[str, str], int]: A message indicating successful deletion,
        or an HTTPException with an appropriate status code.

    Raises:
        HTTPException: An error occurred while trying to delete the result due to the result not being found (404),
                       a validation error occurred (400), or
                       an unspecified error occurred (500).
    """
    try:
        benchmark_result_service.delete_result(result_id)
        return {"message": "Result deleted successfully"}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to delete result: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to delete result: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to delete result: {e.msg}"
            )



def get_paginated_lists(page_size: int, list_of_items: list) -> list:
    """
    Splits a list of items into a list of lists, each containing a maximum of `page_size` items.

    Args:
        page_size (int): The number of items per page.
        list_of_items (list): The list of items to be paginated.

    Returns:
        list: A list of lists, where each sublist contains up to `page_size` items.

    Raises:
        RuntimeError: If `page_size` is not provided or is invalid.
    """

    if not page_size:
        raise RuntimeError("Unable to get page_size for pagination.")

    # Break list_of_items into a list of lists, each of size page_size
    paginated_items = [
        list_of_items[i : i + page_size]
        for i in range(0, len(list_of_items), page_size)
    ]

    return paginated_items



from .... import api as moonshot_api
from ..services.utils.exceptions_handler import exception_handler
from .base_service import BaseService


class ContextStrategyService(BaseService):
    @exception_handler
    def get_ctx_strategy(self) -> list[dict]:
        strategies = moonshot_api.api_get_all_context_strategy_metadata()
        return strategies

    @exception_handler
    def get_ctx_strategy_name(self) -> list[str]:
        strategies = moonshot_api.api_get_all_context_strategies()
        return strategies

    @exception_handler
    def delete_ctx_strategy(self, ctx_strategy_name: str) -> None:
        moonshot_api.api_delete_context_strategy(ctx_strategy_name)



import datetime
import glob
import os
from itertools import chain
from pathlib import Path
from typing import Iterator

import xxhash
from pydantic import ConfigDict, validate_call

from moonshot.src.configs.env_variables import EnvironmentVars, EnvVariables
from moonshot.src.storage.db_interface import DBInterface
from moonshot.src.utils.import_modules import get_instance


class Config:
    config_dict = ConfigDict(arbitrary_types_allowed=True)


class Storage:
    @staticmethod
    @validate_call
    def create_object(
        obj_type: str,
        obj_id: str,
        obj_info: dict,
        obj_extension: str,
        obj_mod_type: str = "jsonio",
    ) -> str:
        """
        Writes the object information to a file.

        Args:
            obj_type (str): The type of the object (e.g., 'recipe', 'cookbook').
            obj_id (str): The ID of the object.
            obj_info (dict): A dictionary containing the object information.
            obj_extension (str): The file extension (e.g., 'json', 'py').
            obj_mod_type (str, optional): The module type for object serialization. Defaults to 'jsonio'.

        Returns:
            str: A filepath string of the object that has just been created.
        """
        if not hasattr(EnvironmentVars, obj_type):
            raise RuntimeError(
                f"'{obj_type}' is not a recognized EnvironmentVar value."
            )

        obj_filepath = Storage.get_filepath(obj_type, obj_id, obj_extension, True)
        if obj_filepath and isinstance(obj_filepath, str):
            obj_mod_instance = get_instance(
                obj_mod_type,
                Storage.get_filepath(EnvVariables.IO_MODULES.name, obj_mod_type, "py"),
            )
            if obj_mod_instance and callable(obj_mod_instance):
                try:
                    obj_mod_instance(obj_filepath).create_file(obj_info)
                    return obj_filepath
                except Exception as e:
                    raise e
            else:
                raise RuntimeError(
                    f"Unable to get defined object module instance - {obj_mod_instance}"
                )
        else:
            raise RuntimeError("Unable to create object.")

    @staticmethod
    @validate_call(config=Config.config_dict)
    def create_object_with_iterator(
        obj_type: str,
        obj_id: str,
        obj_info: dict,
        obj_extension: str,
        obj_mod_type: str = "jsonio",
        iterator_keys: list[str] | None = None,
        iterator_data: Iterator[dict] | None = None,
    ) -> str:
        """
        Writes the object information to a file using iterators for specified keys.

        Args:
            obj_type (str): The type of the object (e.g., 'recipe', 'cookbook').
            obj_id (str): The ID of the object.
            obj_info (dict): A dictionary containing the object information.
            obj_extension (str): The file extension (e.g., 'json', 'py').
            obj_mod_type (str, optional): The module type for object serialization. Defaults to 'jsonio'.
            iterator_keys (list[str] | None): A list of keys for which the values will be written using iterators.
            iterator_data (Iterator[dict] | None): An iterator for the data to be written for the specified keys.

        Returns:
            str: A filepath string of the object that has just been created.
        """
        if not hasattr(EnvironmentVars, obj_type):
            raise RuntimeError(
                f"'{obj_type}' is not a recognized EnvironmentVar value."
            )

        obj_filepath = Storage.get_filepath(obj_type, obj_id, obj_extension, True)
        if not obj_filepath or not isinstance(obj_filepath, str):
            raise RuntimeError("Unable to create object.")

        obj_mod_instance = get_instance(
            obj_mod_type,
            Storage.get_filepath(EnvVariables.IO_MODULES.name, obj_mod_type, "py"),
        )
        if not obj_mod_instance or not callable(obj_mod_instance):
            raise RuntimeError(
                f"Unable to get defined object module instance - {obj_mod_instance}"
            )

        try:
            obj_mod_instance(obj_filepath).create_file_with_iterator(
                obj_info, iterator_keys, iterator_data
            )
            return obj_filepath
        except Exception as e:
            raise e

    @staticmethod
    @validate_call
    def read_object_with_iterator(
        obj_type: str,
        obj_id: str,
        obj_extension: str,
        obj_mod_type: str = "jsonio",
        json_keys: list[str] | None = None,
        iterator_keys: list[str] | None = None,
    ) -> dict:
        """
        Retrieves object data from a file, with options to filter by specific keys for efficient data handling.

        This function reads from a file corresponding to the given object type and ID, deserializing the content
        using the specified module. It can also selectively extract values or create iterators for specified keys,
        which is useful for handling large datasets or deeply nested JSON structures.

        Args:
            obj_type (str): The category of the object to read (e.g., 'recipe', 'cookbook').
            obj_id (str): The unique identifier for the object.
            obj_extension (str): The file extension indicating the file type (e.g., 'json', 'py').
            obj_mod_type (str, optional): The deserialization module type to use. Defaults to 'jsonio'.
            json_keys (list[str] | None, optional): Keys for which to directly extract values from the file.
                If provided, only these keys will be included in the returned dictionary.
            iterator_keys (list[str] | None, optional): Keys for which to create iterators, allowing for
                streamed access to large or nested structures within the file.

        Returns:
            dict: A dictionary with the requested object data. If `json_keys` is provided, the dictionary
            will contain only the specified keys and their values. If `iterator_keys` is provided, the dictionary
            will include iterators for the specified keys, enabling streamed data access.
        """
        if not hasattr(EnvironmentVars, obj_type):
            raise RuntimeError(
                f"'{obj_type}' is not a recognized EnvironmentVar value."
            )

        obj_filepath = Storage.get_filepath(obj_type, obj_id, obj_extension)
        if obj_filepath and isinstance(obj_filepath, str):
            obj_mod_instance = get_instance(
                obj_mod_type,
                Storage.get_filepath(EnvVariables.IO_MODULES.name, obj_mod_type, "py"),
            )
            if obj_mod_instance and callable(obj_mod_instance):
                return obj_mod_instance(obj_filepath).read_file_iterator(
                    json_keys, iterator_keys
                )
            else:
                raise RuntimeError(
                    f"Unable to get defined object module instance - {obj_mod_instance}"
                )
        else:
            raise RuntimeError(f"No {obj_type.lower()} found with ID: {obj_id}")

    @staticmethod
    @validate_call
    def read_object(
        obj_type: str, obj_id: str, obj_extension: str, obj_mod_type: str = "jsonio"
    ) -> dict:
        """
        Reads the object information from a file.

        Args:
            obj_type (str): The type of the object (e.g., 'recipe', 'cookbook').
            obj_id (str): The ID of the object.
            obj_extension (str): The file extension (e.g., 'json', 'py').
            obj_mod_type (str, optional): The module type for object deserialization. Defaults to 'json'.

        Returns:
            dict: A dictionary containing the object information.
        """
        if not hasattr(EnvironmentVars, obj_type):
            raise RuntimeError(
                f"'{obj_type}' is not a recognized EnvironmentVar value."
            )

        obj_filepath = Storage.get_filepath(obj_type, obj_id, obj_extension)
        if obj_filepath and isinstance(obj_filepath, str):
            obj_mod_instance = get_instance(
                obj_mod_type,
                Storage.get_filepath(EnvVariables.IO_MODULES.name, obj_mod_type, "py"),
            )
            if obj_mod_instance and callable(obj_mod_instance):
                return obj_mod_instance(obj_filepath).read_file()
            else:
                raise RuntimeError(
                    f"Unable to get defined object module instance - {obj_mod_instance}"
                )
        else:
            raise RuntimeError(f"No {obj_type.lower()} found with ID: {obj_id}")

    @staticmethod
    @validate_call
    def delete_object(obj_type: str, obj_id: str, obj_extension: str) -> bool:
        """
        Deletes the specified object file.

        Args:
            obj_type (str): The type of the object (e.g., 'recipe', 'cookbook').
            obj_id (str): The ID of the object.
            obj_extension (str): The file extension (e.g., 'json', 'py').

        Returns:
            bool: True if the file was successfully deleted.

        Raises:
            RuntimeError: If no file is found with the specified ID and type.
        """
        obj_filepath = Storage.get_filepath(obj_type, obj_id, obj_extension)
        if obj_filepath:
            Path(obj_filepath).unlink()
            return True
        else:
            raise RuntimeError(f"No {obj_type.lower()} found with ID: {obj_id}")

    @staticmethod
    def count_objects(
        obj_type: str, obj_id: str, obj_extension: str, item_path: str
    ) -> int:
        """
        Counts the number of objects in a dataset without loading them fully into memory.

        Args:
            obj_type (str): The type of the object (e.g., 'dataset').
            obj_id (str): The ID of the object.
            obj_extension (str): The file extension (e.g., 'json').
            item_path (str): The path to the items in the JSON file.

        Returns:
            int: The count of the objects.
        """
        count = 0
        generator = Storage.read_object_with_iterator(
            obj_type, obj_id, obj_extension, iterator_keys=[item_path]
        )
        for _ in generator[item_path.split(".")[0]]:
            count += 1
        return count

    @staticmethod
    @validate_call
    def get_objects(obj_type: str, obj_extension: str) -> Iterator[str]:
        """
        Retrieves all the object files with the specified extension from one or more directories.

        Args:
            obj_type (str): The type of the object (e.g., 'recipe', 'cookbook').
            obj_extension (str): The file extension (e.g., 'json', 'py').

        Returns:
            Iterator[str]: An iterator that yields the filepaths of the object files.
        """
        if not hasattr(EnvironmentVars, obj_type):
            raise RuntimeError(
                f"'{obj_type}' is not a recognized EnvironmentVar value."
            )

        directories = EnvironmentVars.get_file_directory(obj_type)
        return chain.from_iterable(
            glob.iglob(f"{directory}/*.{obj_extension}") for directory in directories
        )

    @staticmethod
    def get_creation_datetime(
        obj_type: str, obj_id: str, obj_extension: str
    ) -> datetime.datetime:
        """
        Retrieves the creation datetime of an object.

        Args:
            obj_type (str): The type of the object (e.g., 'recipe', 'cookbook').
            obj_id (str): The ID of the object.
            obj_extension (str): The file extension (e.g., 'json', 'py').

        Returns:
            datetime: The creation datetime of the object.
        """
        obj_filepath = Storage.get_filepath(obj_type, obj_id, obj_extension)
        if obj_filepath:
            creation_timestamp = os.path.getctime(obj_filepath)
            creation_datetime = datetime.datetime.fromtimestamp(creation_timestamp)
            return creation_datetime
        else:
            raise RuntimeError(f"No {obj_type.lower()} found with ID: {obj_id}")

    @staticmethod
    def get_file_hash(obj_type: str, obj_id: str, obj_extension: str) -> str:
        """
        Retrieves the hash of the file content for an object using the 'xxhash' library, which provides
        an extremely fast non-cryptographic hash algorithm.

        Args:
            obj_type (str): The type of the object (e.g., 'recipe', 'cookbook').
            obj_id (str): The ID of the object.
            obj_extension (str): The file extension (e.g., 'json', 'py').

        Returns:
            str: The hex digest of the xxHash of the file content.
        """
        obj_filepath = Storage.get_filepath(obj_type, obj_id, obj_extension)
        if obj_filepath and Path(obj_filepath).exists():
            with open(obj_filepath, "rb") as file:
                file_content = file.read()
            file_hash = xxhash.xxh64(file_content).hexdigest()
            return file_hash
        else:
            raise RuntimeError(f"No {obj_type.lower()} found with ID: {obj_id}")

    @staticmethod
    def get_filepath(
        obj_type: str, obj_id: str, obj_extension: str, ignore_existance: bool = False
    ) -> str:
        """
        Retrieves the file path for an object.

        This method uses the provided object type, object ID, and object extension to construct the file path
        of the object. If the creation flag is set to True, it returns the file path even if the file does not exist.

        Args:
            obj_type (str): The type of the object (e.g., 'recipe', 'cookbook').
            obj_id (str): The ID of the object.
            obj_extension (str): The file extension (e.g., 'json', 'py').
            ignore_existance (bool, optional): A flag indicating whether to return the file path
                                        even if the file does not exist. Defaults to False.

        Returns:
            str: The file path of the object.
        """
        return EnvironmentVars.get_file_path(
            obj_type, f"{obj_id}.{obj_extension}", ignore_existance
        )

    @staticmethod
    @validate_call
    def is_object_exists(obj_type: str, obj_id: str, obj_extension: str) -> bool:
        """
        Checks if the object exists.

        Args:
            obj_type (str): The type of the object (e.g., 'runner', 'recipe', 'cookbook').
            obj_id (str): The ID of the object.
            obj_extension (str): The file extension (e.g., 'json', 'py').

        Returns:
            bool: True if the object exists, False otherwise.
        """
        filepath = Storage.get_filepath(obj_type, obj_id, obj_extension)
        if filepath:
            return Path(filepath).exists()
        else:
            return False

    @staticmethod
    def create_database_connection(
        obj_type: str, obj_id: str, obj_extension: str, db_mod_type: str = "sqlite"
    ) -> DBInterface:
        """
        Establishes a database connection for a specific object.

        Args:
            obj_type (str): The type of the object for which the database connection is to be established
            (e.g., 'runner', 'recipe', 'cookbook').

            obj_id (str): The unique identifier of the object for which the database connection is to be established.

            obj_extension (str): The file extension of the object for which the database connection is to be established
            (e.g., 'json', 'py').

            db_mod_type (str): The type of database module to use for establishing the connection (e.g., 'sqlite).
            Defaults to 'sqlite'.

        Returns:
            DBInterface: An instance of the database accessor, which can be used to interact with the database.
        """
        database_instance = get_instance(
            db_mod_type,
            Storage.get_filepath(
                EnvVariables.DATABASES_MODULES.name, db_mod_type, "py"
            ),
        )
        if database_instance:
            database_instance = database_instance(
                Path(Storage.get_filepath(obj_type, obj_id, obj_extension, True))
            )
            if database_instance.create_connection():
                return database_instance
            else:
                raise RuntimeError(f"Failed to create connection - {db_mod_type}")
        else:
            raise RuntimeError(
                f"Unable to get defined database instance - {db_mod_type}"
            )

    @staticmethod
    def close_database_connection(database_instance: DBInterface) -> None:
        """
        Closes the database connection.

        Args:
            database_instance (DBInterface): The instance of the database accessor.

        Returns:
            None
        """
        if database_instance:
            database_instance.close_connection()
        else:
            raise RuntimeError("Database instance is not initialised.")

    @staticmethod
    def create_database_table(
        database_instance: DBInterface, sql_create_table: str
    ) -> None:
        """
        Creates a table in the database.

        This method is used to create a table in the database. If the database instance is not initialised,
        it raises a RuntimeError. Otherwise, it calls the create_table method of the database instance.

        Args:
            database_instance (DBInterface): The database accessor instance.
            sql_create_table (str): The SQL query to create a table.

        Returns:
            None
        """
        if database_instance:
            database_instance.create_table(sql_create_table)
        else:
            raise RuntimeError("Database instance is not initialised.")

    @staticmethod
    def create_database_record(
        database_instance: DBInterface, data: tuple, sql_create_record: str
    ) -> tuple | None:
        """
        Inserts a record into the database.

        This method is used to insert a record into the database. If the database instance is not initialised,
        it raises a RuntimeError. If the database instance is initialised, it calls the create_record method of the
        database instance with the provided data and SQL query.

        Args:
            database_instance (DBInterface): The database accessor instance.
            data (tuple): The data to be inserted into the database.
            sql_create_record (str): The SQL query to insert a record.

        Returns:
            tuple | None: The inserted record if successful, None otherwise.
        """
        if database_instance:
            return database_instance.create_record(data, sql_create_record)
        else:
            raise RuntimeError("Database instance is not initialised.")

    @staticmethod
    def read_database_record(
        database_instance: DBInterface, data: tuple, sql_read_record: str
    ) -> tuple | None:
        """
        Reads a record from the database.

        This method is used to retrieve a record from the database. If the database instance is not initialised,
        it raises a RuntimeError. If the database instance is initialised, it calls the read_record method of the
        database instance and returns the record if found.

        Args:
            database_instance (DBInterface): The database accessor instance.
            data (tuple): The data to be matched for reading the record.
            sql_read_records (str): The SQL query to read a record.

        Returns:
            tuple | None: The record if found, None otherwise.
        """
        if database_instance:
            return database_instance.read_record(data, sql_read_record)
        else:
            raise RuntimeError("Database instance is not initialised.")

    @staticmethod
    def read_database_records(
        database_instance: DBInterface, sql_read_records: str
    ) -> list[tuple] | None:
        """
        Reads records from the database.

        This method is used to retrieve records from the database. If the database instance is not initialised,
        it raises a RuntimeError. If the database instance is initialised, it calls the read_records method of the
        database instance and returns the record if found.

        Args:
            database_instance (DBInterface): The database accessor instance.
            data (tuple): The data to be matched for reading the record.
            sql_read_records (str): The SQL query to read a record.

        Returns:
            tuple | None: The record if found, None otherwise.
        """
        if database_instance:
            return database_instance.read_records(sql_read_records)
        else:
            raise RuntimeError("Database instance is not initialised.")

    @staticmethod
    def update_database_record(
        database_instance: DBInterface, data: tuple, sql_update_record: str
    ) -> None:
        """
        Updates a record in the database.

        This method is used to update a record in the database. If the database instance is not initialised,
        it raises a RuntimeError. Otherwise, it calls the update_record method of the database instance.

        Args:
            database_instance (DBInterface): The database accessor instance.
            data (tuple): The data to be updated.
            sql_update_record (str): The SQL query to update a record.

        Returns:
            None
        """
        if database_instance:
            database_instance.update_record(data, sql_update_record)
        else:
            raise RuntimeError("Database instance is not initialised.")

    @staticmethod
    def delete_database_record_in_table(
        database_instance: DBInterface, sql_delete_record: str
    ) -> None:
        """
        Deletes records from a table in the database based on a SQL condition.

        This method is used to delete records from a specific table in the database
            that meet the condition specified in the SQL delete statement.
        If the database instance is not initialised, it raises a RuntimeError.
        Otherwise, it calls the delete_records_in_table method of the database instance with the provided SQL query.

        Args:
            database_instance (DBInterface): The database accessor instance.
            sql_delete_record (str): The SQL query to delete records from a table.

        Returns:
            None
        """
        if database_instance:
            database_instance.delete_records_in_table(sql_delete_record)
        else:
            raise RuntimeError("Database instance is not initialised.")

    @staticmethod
    def check_database_table_exists(
        database_instance: DBInterface, table_name: str
    ) -> bool | None:
        """
        Checks if a table exists in the database.

        This method checks if the specified table exists in the database. If the database instance is not initialised,
        it raises a RuntimeError.

        Args:
            database_instance (DBInterface): The database accessor instance.
            table_name (str): The name of the table to check for existence.

        Returns:
            bool | None: True if the table exists, False if it does not, None if the database
            instance is not initialised.
        """
        if database_instance:
            return database_instance.check_database_table_exists(table_name)
        else:
            raise RuntimeError("Database instance is not initialised.")

    @staticmethod
    def delete_database_table(
        database_instance: DBInterface, sql_delete_table: str
    ) -> None:
        """
        Deletes a table from the database.

        This method is used to delete a table from the database. If the database instance is not initialised,
        it raises a RuntimeError. Otherwise, it calls the delete_database_table method of the database instance.

        Args:
            database_instance (DBInterface): The database accessor instance.
            sql_delete_table (str): The SQL query to delete a table.

        Returns:
            None
        """
        if database_instance:
            database_instance.delete_database_table(sql_delete_table)
        else:
            raise RuntimeError("Database instance is not initialised.")



def transform_web_format(
    original_dict: dict[str, dict[str, list[dict[str, dict[str, float]]]]]
) -> dict[str, dict[str, list[dict[str, list[dict[str, list[dict[str, float]]]]]]]]:
    transformed = {"metadata": original_dict["metadata"], "results": {"cookbooks": []}}

    for cookbook_name, cookbook_data in original_dict.items():
        if cookbook_name == "metadata":
            continue  # Skip the metadata entry

        cookbook_entry = {"id": cookbook_name, "recipes": []}
        for recipe_name, recipe_data in cookbook_data.items():
            recipe_entry = {"id": recipe_name, "models": []}

            for model_data_str, model_data in recipe_data.items():
                # Extract model, recipe, dataset, prompt_template from the string
                model, _, dataset, prompt_template = eval(model_data_str)

                model_entry = {
                    "id": model,
                    "datasets": [
                        {
                            "id": dataset,
                            "prompt_templates": [
                                {
                                    "id": prompt_template,
                                    "metrics": model_data["results"],
                                }
                            ],
                        }
                    ],
                }

                # Check if the model already exists in the recipe_entry
                existing_model = next(
                    (m for m in recipe_entry["models"] if m["id"] == model), None
                )
                if existing_model:
                    # Check if the dataset already exists in the model
                    existing_dataset = next(
                        (d for d in existing_model["datasets"] if d["id"] == dataset),
                        None,
                    )
                    if existing_dataset:
                        existing_dataset["prompt_templates"].append(
                            model_entry["datasets"][0]["prompt_templates"][0]
                        )
                    else:
                        existing_model["datasets"].append(model_entry["datasets"][0])
                else:
                    recipe_entry["models"].append(model_entry)

            cookbook_entry["recipes"].append(recipe_entry)

        transformed["results"]["cookbooks"].append(cookbook_entry)

    return transformed



from __future__ import annotations

from pydantic import BaseModel

from moonshot.src.runs.run_status import RunStatus


class ResultArguments(BaseModel):
    id: str  # The ID of the Runner.

    start_time: float  # The start time of the Run.

    end_time: float  # The end time of the Run.

    duration: int  # The duration of the Run.

    status: RunStatus  # Status of the Run.

    raw_results: dict = {}  # Raw Results of the Run from runners-modules.

    results: dict = {}  # Results of the Run from results-modules.

    params: dict = {}  # Other information required for results module

    def to_dict(self) -> dict:
        """
        Transforms the ResultArguments instance into a dictionary format.

        This method serializes the ResultArguments instance into a dictionary where attribute names become keys
        and their corresponding values are the dictionary values.

        Returns:
            A dictionary representation of the ResultArguments instance.
        """
        return {
            "id": self.id,
            "start_time": self.start_time,
            "end_time": self.end_time,
            "duration": self.duration,
            "status": self.status.name,
            "raw_results": self.raw_results,
            "results": self.results,
            "params": self.params,
        }



from moonshot.src.connectors_endpoints.connector_endpoint_arguments import (
    ConnectorEndpointArguments as ConnectorEndpointPydanticModel,
)


class EndpointDataModel(ConnectorEndpointPydanticModel):
    pass

    def mask_token(self):
        if self.token:
            self.token = "*" * len(self.token)



from pydantic import validate_call

from moonshot.src.redteaming.attack.context_strategy import ContextStrategy


# ------------------------------------------------------------------------------
# Context Strategy APIs
# ------------------------------------------------------------------------------
def api_get_all_context_strategies() -> list[str]:
    """
    Retrieves and returns the names of all context strategies currently available.

    This API endpoint interfaces with the `ContextStrategy.get_all_context_strategies` method to fetch a list
    of all context strategy names. It's designed for clients that need to know what context strategies are available for
    use in sessions or other components of the system.

    Returns:
        list[str]: A list of strings, each representing the name of a context strategy.
    """
    return ContextStrategy.get_all_context_strategies()


@validate_call
def api_delete_context_strategy(cs_id: str) -> bool:
    """
    Deletes a context strategy identified by its ID.

    This API endpoint interfaces with the `ContextStrategy.delete` method to remove a context strategy from the system.
    It is used to manage the available context strategies by allowing for their removal when they are no longer needed.

    Args:
        cs_id (str): The unique identifier of the context strategy to be deleted.

    Returns:
        bool: True if the context strategy was successfully deleted.

    Raises:
        Exception: If the deletion process encounters an error.
    """
    return ContextStrategy.delete(cs_id)


def api_get_all_context_strategy_metadata() -> list[dict]:
    """
    Retrieves metadata for all context strategy modules.

    This function retrieves the metadata for all available context strategies and
    returns a list of metadata dictionaries.

    Returns:
        list[dict]: A list of dictionaries, each representing the details of a context strategy's metadata.
    """

    return [
        ContextStrategy.load(context_strategy_name).get_metadata()  # type: ignore ; ducktyping
        for context_strategy_name in ContextStrategy.get_all_context_strategies()
    ]



from abc import abstractmethod
from typing import Any

from moonshot.src.configs.env_variables import EnvVariables
from moonshot.src.storage.storage import Storage
from moonshot.src.utils.log import configure_logger
from moonshot.src.utils.timeit import timeit

# Create a logger for this module
logger = configure_logger(__name__)


class MetricInterface:
    config_name = "metrics_config"
    config_extension = "json"

    @abstractmethod
    @timeit
    def get_metadata(self) -> dict | None:
        """
        Abstract method to retrieve metadata from a metric class.

        Returns:
            dict | None: Returns a dictionary containing the metadata of the metric class, or None if the operation was
            unsuccessful.
        """
        pass

    @abstractmethod
    async def get_results(
        self, prompts: Any, predicted_results: Any, targets: Any, *args, **kwargs
    ) -> dict:
        """
        Abstract method to compute metrics based on model predictions.

        This method is intended to be implemented by subclasses to calculate specific
        metrics (e.g., accuracy, BLEU score) based on the inputs provided.
        It should handle any necessary preprocessing of inputs, computation of the metric, and formatting of the output.

        Args:
            prompts (Any): The input prompts used to generate the predictions.
            The exact type and format will depend on the specific implementation and the nature of the
            task (e.g., text, images).

            predicted_results (Any): The model's predictions in response to the prompts.
            The type and format should align with the prompts and the expected output of the model.

            targets (Any): The ground truth targets against which the predicted results will be evaluated.

            *args: Variable length argument list for additional parameters that might be required for specific metric
            computations.

            **kwargs: Arbitrary keyword arguments for additional options or configuration settings that might be
            required for specific metric computations.

        Returns:
            dict: A dictionary containing the computed metric(s). The keys should clearly indicate the metric being
            reported, and the values should be the corresponding metric values.
        """
        pass

    def get_metrics_configuration(self, met_id: str) -> dict:
        """
        Retrieves the configuration for a specific metric by its identifier.

        Args:
            met_id (str): The identifier for the metric configuration to retrieve.

        Returns:
            dict: The metric configuration as a dictionary. Returns an empty dict if the configuration is not found.

        Raises:
            Exception: If reading the metrics configuration fails or if the configuration cannot be created.
        """
        try:
            obj_results = Storage.read_object(
                EnvVariables.METRICS.name, "metrics_config", "json"
            )
            return obj_results.get(met_id, {})

        except Exception as e:
            logger.warning(
                f"[MetricInterface] Failed to read metrics configuration: {str(e)}"
            )
            logger.info("Attempting to create empty metrics configuration...")
            try:
                Storage.create_object(
                    obj_type=EnvVariables.METRICS.name,
                    obj_id="metrics_config",
                    obj_info={},
                    obj_extension="json",
                )
                # After creation, attempt to read it again to ensure it was created successfully
                obj_results = Storage.read_object(
                    EnvVariables.METRICS.name, "metrics_config", "json"
                )
                return obj_results.get(met_id, {})
            except Exception as e:
                raise Exception(
                    f"[MetricInterface] Failed to create metrics configuration: {str(e)}"
                )



from typing import Optional

from moonshot.src.bookmark.bookmark_arguments import (
    BookmarkArguments as BookmarkPydanticModel,
)


class BookmarkCreateDTO(BookmarkPydanticModel):
    prompt_template: Optional[str] = ""
    context_strategy: Optional[str] = ""
    attack_module: Optional[str] = ""
    metric: Optional[str] = ""
    bookmark_time: Optional[str] = None



from dependency_injector.wiring import Provide, inject
from fastapi import APIRouter, Depends, HTTPException

from ..container import Container
from ..services.context_strategy_service import ContextStrategyService
from ..services.utils.exceptions_handler import ServiceException

router = APIRouter(tags=["Context Strategy"])


@router.get("/api/v1/context-strategies")
@inject
def get_all_context_strategies(
    context_strategy_service: ContextStrategyService = Depends(
        Provide[Container.context_strategy_service]
    ),
) -> list[dict]:
    """
    Retrieve all context strategies from the database.

    Args:
        context_strategy_service (ContextStrategyService): The service responsible for retrieving context strategies.

    Returns:
        list[dict]: A list of context strategies with details.

    Raises:
        HTTPException: An error with status code 404 if no context strategies are found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return context_strategy_service.get_ctx_strategy()
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404,
                detail=f"Failed to retrieve context strategies: {e.msg}",
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400,
                detail=f"Failed to retrieve context strategies: {e.msg}",
            )
        else:
            raise HTTPException(
                status_code=500,
                detail=f"Failed to retrieve context strategies: {e.msg}",
            )


@router.get("/api/v1/context-strategies/name")
@inject
def get_all_context_strategies_name(
    context_strategy_service: ContextStrategyService = Depends(
        Provide[Container.context_strategy_service]
    ),
) -> list[str]:
    """
    Retrieve all context strategies from the database.

    Args:
        context_strategy_service (ContextStrategyService): The service responsible for retrieving context strategies.

    Returns:
        list[str]: A list of context strategies.

    Raises:
        HTTPException: An error with status code 404 if no context strategies are found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return context_strategy_service.get_ctx_strategy_name()
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404,
                detail=f"Failed to retrieve context strategies: {e.msg}",
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400,
                detail=f"Failed to retrieve context strategies: {e.msg}",
            )
        else:
            raise HTTPException(
                status_code=500,
                detail=f"Failed to retrieve context strategies: {e.msg}",
            )


@router.delete("/api/v1/context-strategies/{ctx_strategy_name}")
@inject
def delete_context_strategy(
    ctx_strategy_name: str,
    context_strategy_service: ContextStrategyService = Depends(
        Provide[Container.context_strategy_service]
    ),
) -> dict[str, bool]:
    """
    Delete a context strategy from the database by its name.

    Args:
        ctx_strategy_name (str): The name of the context strategy to delete.
        context_strategy_service (ContextStrategyService): The service responsible for deleting the context strategy.

    Returns:
        dict[str, bool]: A dictionary with a key 'success' indicating the result of the deletion operation.

    Raises:
        HTTPException: An error with status code 404 if the context strategy is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        context_strategy_service.delete_ctx_strategy(ctx_strategy_name)
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to delete context strategy: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to delete context strategy: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to delete context strategy: {e.msg}"
            )



from .... import api as moonshot_api
from ..schemas.dataset_create_dto import CSV_Dataset_DTO, HF_Dataset_DTO
from ..schemas.dataset_response_dto import DatasetResponseDTO
from ..services.base_service import BaseService
from ..services.utils.exceptions_handler import exception_handler
from .utils.file_manager import copy_file
import os


class DatasetService(BaseService):
    @exception_handler
    def convert_dataset(self, dataset_data: CSV_Dataset_DTO) -> str:
        """
        Convert a dataset using the provided dataset data.

        Args:
            dataset_data (CSV_Dataset_DTO): The data required to convert the dataset.

        Returns:
            str: The filename of the newly created dataset.

        Raises:
            Exception: If an error occurs during dataset conversion.
        """

        new_ds_path = moonshot_api.api_convert_dataset(
            name=dataset_data.name,
            description=dataset_data.description,
            reference=dataset_data.reference,
            license=dataset_data.license,
            file_path=dataset_data.file_path,
        )
        return os.path.splitext(os.path.basename(new_ds_path))[0]

    @exception_handler
    def download_dataset(self, dataset_data: HF_Dataset_DTO) -> str:
        """
        Download a dataset using the provided dataset data.

        Args:
            dataset_data (HF_Dataset_DTO): The data required to download the dataset.

        Returns:
            str: The path to the newly downloaded dataset.

        Raises:
            Exception: If an error occurs during dataset download.
        """

        new_ds_path = moonshot_api.api_download_dataset(
            name=dataset_data.name,
            description=dataset_data.description,
            reference=dataset_data.reference,
            license=dataset_data.license,
            **dataset_data.params,
        )
        return copy_file(new_ds_path)

    @exception_handler
    def get_all_datasets(self) -> list[DatasetResponseDTO]:
        datasets = moonshot_api.api_get_all_datasets()
        return [DatasetResponseDTO.model_validate(dataset) for dataset in datasets]

    @exception_handler
    def get_all_datasets_name(self) -> list[str] | None:
        datasets = moonshot_api.api_get_all_datasets_name()
        return datasets

    @exception_handler
    def delete_dataset(self, dataset_id: str) -> None:
        moonshot_api.api_delete_dataset(dataset_id)



from abc import ABC, abstractmethod

from ...types.types import TestRunProgress


class InterfaceBenchmarkProgressCallback(ABC):
    """
    The interface to implement for handling of benchmark progress updates from moonshot library

    """

    @abstractmethod
    def on_progress_update(self, progress_data: TestRunProgress) -> None:
        pass



from ast import literal_eval

import cmd2
from rich.console import Console
from rich.table import Table

from moonshot.api import api_delete_metric, api_get_all_metric, api_get_all_metric_name
from moonshot.integrations.cli.cli_errors import (
    ERROR_BENCHMARK_DELETE_METRIC_METRIC_VALIDATION,
    ERROR_BENCHMARK_LIST_METRICS_FIND_VALIDATION,
    ERROR_BENCHMARK_LIST_METRICS_PAGINATION_VALIDATION,
    ERROR_BENCHMARK_LIST_METRICS_PAGINATION_VALIDATION_1,
    ERROR_BENCHMARK_VIEW_METRIC_METRIC_FILENAME_VALIDATION,
)
from moonshot.integrations.cli.utils.process_data import filter_data

console = Console()


# ------------------------------------------------------------------------------
# CLI Functions
# ------------------------------------------------------------------------------
def list_metrics(args) -> list | None:
    """
    List all available metrics.

    This function retrieves all available metrics by calling the api_get_all_metric function from the
    moonshot.api module. It then filters the metrics based on the provided keyword and pagination arguments.
    If there are no metrics, it prints a message indicating that no metrics were found.

    Args:
        args: A namespace object from argparse. It should have optional attributes:
            find (str): Optional field to find metric(s) with a keyword.
            pagination (str): Optional field to paginate metrics.

    Returns:
        list | None: A list of metrics or None if there are no metrics.
    """

    try:
        print("Listing metrics may take a while...")
        if args.find is not None:
            if not isinstance(args.find, str) or not args.find:
                raise TypeError(ERROR_BENCHMARK_LIST_METRICS_FIND_VALIDATION)

        if args.pagination is not None:
            if not isinstance(args.pagination, str) or not args.pagination:
                raise TypeError(ERROR_BENCHMARK_LIST_METRICS_PAGINATION_VALIDATION)
            try:
                pagination = literal_eval(args.pagination)
                if not (
                    isinstance(pagination, tuple)
                    and len(pagination) == 2
                    and all(isinstance(i, int) for i in pagination)
                ):
                    raise ValueError(
                        ERROR_BENCHMARK_LIST_METRICS_PAGINATION_VALIDATION_1
                    )
            except (ValueError, SyntaxError):
                raise ValueError(ERROR_BENCHMARK_LIST_METRICS_PAGINATION_VALIDATION_1)
        else:
            pagination = ()

        metrics_list = api_get_all_metric()
        keyword = args.find.lower() if args.find else ""

        if metrics_list:
            filtered_metrics_list = filter_data(metrics_list, keyword, pagination)
            if filtered_metrics_list:
                _display_metrics(filtered_metrics_list)
                return filtered_metrics_list

        console.print("[red]There are no metrics found.[/red]")
        return None

    except Exception as e:
        print(f"[list_metrics]: {str(e)}")
        return None


def view_metric(args) -> None:
    """
    View a specific metric.

    This function retrieves all available metrics and their names by calling the api_get_all_metric and
    api_get_all_metric_name functions. It then finds the metric with the name specified in args.metric_filename
    and displays it using the _display_metrics function. If an exception occurs, it prints an error message.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            metric_filename (str): The name of the metric to view.

    Returns:
        None
    """
    try:
        print("Viewing metrics may take a while...")
        if (
            not isinstance(args.metric_filename, str)
            or not args.metric_filename
            or args.metric_filename is None
        ):
            raise TypeError(ERROR_BENCHMARK_VIEW_METRIC_METRIC_FILENAME_VALIDATION)

        metrics_list = api_get_all_metric()
        metrics_name_list = api_get_all_metric_name()

        # Find the index of the metric with the name args.metric_filename
        metric_index = metrics_name_list.index(args.metric_filename)
        # Pass the corresponding metric from metrics_list to _display_metrics
        _display_metrics([metrics_list[metric_index]])

    except Exception as e:
        print(f"[view_metric]: {str(e)}")


def delete_metric(args) -> None:
    """
    Delete a metric.

    This function deletes a metric with the specified identifier. It prompts the user for confirmation before proceeding
    with the deletion. If the user confirms, it calls the api_delete_metric function from the moonshot.api module to
    delete the metric. If the deletion is successful, it prints a confirmation message. If an exception occurs, it
    prints an error message.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            metric (str): The identifier of the metric to delete.

    Returns:
        None
    """
    # Confirm with the user before deleting a metric
    confirmation = console.input(
        "[bold red]Are you sure you want to delete the metric (y/N)? [/]"
    )
    if confirmation.lower() != "y":
        console.print("[bold yellow]Metric deletion cancelled.[/]")
        return

    try:
        if args.metric is None or not isinstance(args.metric, str) or not args.metric:
            raise ValueError(ERROR_BENCHMARK_DELETE_METRIC_METRIC_VALIDATION)

        api_delete_metric(args.metric)
        print("[delete_metric]: Metric deleted.")
    except Exception as e:
        print(f"[delete_metric]: {str(e)}")


# ------------------------------------------------------------------------------
# Helper functions: Display on cli
# ------------------------------------------------------------------------------
def _display_metrics(metrics_list: list):
    """
    Displays a list of metrics in a table format.

    This function takes a list of metrics and displays them in a table format with each metric's ID, name, and
    description. If the list is empty, it prints a message indicating that no metrics are found.

    Args:
        metrics_list (list): A list of dictionaries, where each dictionary contains the details of a metric.

    Returns:
        None
    """
    table = Table(
        title="List of Metrics", show_lines=True, expand=True, header_style="bold"
    )
    table.add_column("No.", width=2)
    table.add_column("Metric", justify="left", width=78)
    for idx, metric in enumerate(metrics_list, 1):
        id, name, description, endpoints, configurations, *other_args = metric.values()
        idx = metric.get("idx", idx)
        result_info = (
            f"[red]Id: {id}[/red]\n\n"
            f"[blue]Name: {name}[/blue]\n\n"
            f"[blue]Description: {description}[/blue]\n\n"
            f"[blue]Endpoints: {endpoints}[/blue]\n\n"
            f"[blue]Configurations:{configurations}[/blue]\n\n"
        )

        table.add_section()
        table.add_row(str(idx), result_info)
    console.print(table)


# ------------------------------------------------------------------------------
# Cmd2 Arguments Parsers
# ------------------------------------------------------------------------------
# View metric arguments
view_metric_args = cmd2.Cmd2ArgumentParser(
    description="View a metric file.",
    epilog="Example:\n view_metric my-new-metric",
)
view_metric_args.add_argument(
    "metric_filename", type=str, help="Name of the metric file"
)

# Delete metric arguments
delete_metric_args = cmd2.Cmd2ArgumentParser(
    description="Delete a metric.",
    epilog="Example:\n delete_metric my-new-metric",
)
delete_metric_args.add_argument("metric", type=str, help="Name of the metric")

# List metric arguments
list_metrics_args = cmd2.Cmd2ArgumentParser(
    description="List all metrics.",
    epilog='Example:\n list_metrics -f "exact"',
)

list_metrics_args.add_argument(
    "-f",
    "--find",
    type=str,
    help="Optional field to find metric(s) with keyword",
    nargs="?",
)

list_metrics_args.add_argument(
    "-p",
    "--pagination",
    type=str,
    help="Optional tuple to paginate metric(s). E.g. (2,10) returns 2nd page with 10 items in each page.",
    nargs="?",
)



from typing import Any

from pydantic import BaseModel, Field


class RecipeArguments(BaseModel):
    # id (str): The ID of the recipe.
    id: str

    name: str = Field(min_length=1)  # name (str): The name for the endpoint.

    # description (str): The description of the recipe.
    description: str

    # tags (list): The list of tags in the recipe.
    tags: list[str]

    # categories (list): The list of categories in the recipe.
    categories: list[str]

    # datasets (list): The list of datasets used in the recipe.
    datasets: list[str] = Field(min_length=1)

    # prompt_templates (list): The list of prompt templates in the recipe.
    prompt_templates: list[str]

    # metrics (list): The list of metrics in the recipe.
    metrics: list[str] = Field(min_length=1)

    # grading_scale (dict): A dictionary where keys are grading categories and values are lists of grading scale.
    grading_scale: dict[str, list[int]]

    # stats (dict): A dictionary containing statistics about the recipe.
    stats: dict = {}

    def __init__(self, **data: Any):
        super().__init__(**data)
        self.validate_grading_scale()

    def get_start_of_grading_scale(self, item: tuple[str, list[int]]) -> int:
        """
        Retrieve the starting value of a grading scale for a given grade.

        Args:
            item (tuple[str, list[int]]): A tuple containing the grade as a string and the associated grading scale
            as a list of ints.

        Returns:
            int: The first value in the grading scale list, representing the start of the grading range.
        """
        _, grading_scale = item
        return grading_scale[0]

    def validate_grading_scale(self) -> None:
        """
        Validate the grading scale to ensure that it covers a continuous range from 0 to 100.

        This method checks that each grading range starts where the previous one ended and that
        the final grading range ends at 100. If any grading range does not start as expected or
        if the end of the last grading range is not 100, it raises a ValueError.

        Raises:
            ValueError: If any grading range does not start as expected or if the grading ranges
            do not cover the full range from 0 to 100.
        """
        if not self.grading_scale:  # Check if grading_scale is empty
            return  # If empty, it's considered valid and the method returns early

        expected_start = 0
        for grade, grading_range in sorted(
            self.grading_scale.items(), key=self.get_start_of_grading_scale
        ):
            start, end = grading_range
            if start != expected_start:
                raise ValueError(
                    f"Invalid grading range for '{grade}'. Expected start: {expected_start}, got {start}."
                )
            if end < start:
                raise ValueError(
                    f"Invalid grading range for '{grade}'. The end value {end} is less than "
                    f"the start value {start}."
                )
            expected_start = end + 1

        if expected_start - 1 != 100:
            raise ValueError("Grading ranges do not cover 0 to 100.")

    def to_dict(self) -> dict:
        """
        Convert the RecipeArguments object to a dictionary.

        Returns:
            dict: A dictionary representation of the RecipeArguments object.
            The keys are the attribute names and the values are the attribute values.
        """
        return {
            "id": self.id,
            "name": self.name,
            "description": self.description,
            "tags": self.tags,
            "categories": self.categories,
            "datasets": self.datasets,
            "prompt_templates": self.prompt_templates,
            "metrics": self.metrics,
            "grading_scale": self.grading_scale,
            "stats": self.stats,
        }



from dependency_injector.wiring import Provide, inject
from fastapi import APIRouter, Depends, HTTPException

from ..container import Container
from ..services.runner_service import RunnerService
from ..services.utils.exceptions_handler import ServiceException

router = APIRouter(tags=["Runner"])


@router.get("/api/v1/runners")
@inject
def get_all_runners(
    runner_service: RunnerService = Depends(Provide[Container.runner_service]),
) -> list:
    """
    Retrieve all runners from the database.

    Args:
        runner_service (RunnerService): The service responsible for retrieving runners.

    Returns:
        list: A list of all runners.

    Raises:
        HTTPException: An error with status code 404 if no runners are found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return runner_service.get_all_runner()
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve runners: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve runners: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve runners: {e.msg}"
            )


@router.get("/api/v1/runners/name")
@inject
def get_all_runner_name(
    runner_service: RunnerService = Depends(Provide[Container.runner_service]),
) -> list:
    """
    Retrieve all runner names from the database.

    Args:
        runner_service (RunnerService): The service responsible for retrieving runner names.

    Returns:
        list: A list of all runner names.

    Raises:
        HTTPException: An error with status code 404 if no runner names are found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return runner_service.get_all_runner_name()
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve runner names: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve runner names: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve runner names: {e.msg}"
            )


@router.get("/api/v1/runners/{runner_id}")
@inject
def get_runner_by_id(
    runner_id: str,
    runner_service: RunnerService = Depends(Provide[Container.runner_service]),
) -> dict:
    """
    Retrieve a specific runner by their ID from the database.

    Args:
        runner_id (str): The unique identifier of the runner to retrieve.
        runner_service (RunnerService): The service responsible for retrieving the runner.

    Returns:
        dict: The runner information if found.

    Raises:
        HTTPException: An error with status code 404 if the runner is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        run = runner_service.get_runner_by_id(runner_id)
        return run
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve runner: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve runner: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve runner: {e.msg}"
            )


@router.delete("/api/v1/runners/{runner_id}")
@inject
def delete_runner(
    runner_id: str,
    runner_service: RunnerService = Depends(Provide[Container.runner_service]),
) -> dict:
    """
    Delete a specific runner by their ID from the database.

    Args:
        runner_id (str): The unique identifier of the runner to delete.
        runner_service (RunnerService): The service responsible for deleting the runner.

    Returns:
        dict: A message indicating the successful deletion of the runner.

    Raises:
        HTTPException: An error with status code 404 if the runner is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        runner_service.delete_run(runner_id)
        return {"message": "Runner deleted successfully"}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to delete runner: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to delete runner: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to delete runner: {e.msg}"
            )


@router.get("/api/v1/runners/{runner_id}/runs/{run_id}")
@inject
def get_run_details_by_runner(
    runner_id: str,
    run_id: str,
    runner_service: RunnerService = Depends(Provide[Container.runner_service]),
) -> dict:
    """
    Retrieve the details of a specific run by a runner.

    Args:
        runner_id (str): The unique identifier of the runner.
        run_id (str): The unique identifier of the run.
        runner_service (RunnerService): The service responsible for retrieving run details.

    Returns:
        dict: The details of the run.

    Raises:
        HTTPException: An error with status code 404 if the run details are not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return runner_service.get_run_details_by_runner(runner_id, run_id)
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404,
                detail=f"Failed to get run details from runner: {e.msg}",
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400,
                detail=f"Failed to get run details from runner: {e.msg}",
            )
        else:
            raise HTTPException(
                status_code=500,
                detail=f"Failed to get run details from runner: {e.msg}",
            )


@router.get("/api/v1/runners/{runner_id}/runs")
@inject
def get_runs_id_in_runner(
    runner_id: str,
    runner_service: RunnerService = Depends(Provide[Container.runner_service]),
) -> list[int]:
    """
    Retrieve a list of run identifiers associated with a specific runner.

    Args:
        runner_id (str): The unique identifier of the runner.
        runner_service (RunnerService): The service responsible for retrieving the list of runs.

    Returns:
        List[str]: A list of run identifiers.

    Raises:
        HTTPException: An error with status code 404 if no runs are found for the runner.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return runner_service.get_runs_id_in_runner(runner_id)
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to get runs from runner: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to get runs from runner: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to get runs from runner: {e.msg}"
            )



import argparse
import os
import platform
import shutil
import subprocess
import sys
import threading
import warnings
from typing import Any

from dotenv import dotenv_values

from moonshot.api import api_set_environment_variables
from moonshot.src.utils.log import configure_logger

# Create a logger for this module
logger = configure_logger(__name__)

"""
Run the Moonshot application
"""


def run_subprocess(*args: Any, **kwargs: Any) -> subprocess.CompletedProcess:
    """
    Run a subprocess with the option to use shell=True on Windows.
    """
    if platform.system() == "Windows":
        kwargs["shell"] = True
    return subprocess.run(*args, **kwargs)


def ms_lib_env_file(data_repo_name: str) -> None:
    """
    Writes the env file to be used for moonshot library
    """
    env_content_data = f"""
    # For Data
    ATTACK_MODULES="./{data_repo_name}/attack-modules"
    BOOKMARKS="./{data_repo_name}/generated-outputs/bookmarks"
    CONNECTORS="./{data_repo_name}/connectors"
    CONNECTORS_ENDPOINTS="./{data_repo_name}/connectors-endpoints"
    CONTEXT_STRATEGY="./{data_repo_name}/context-strategy"
    COOKBOOKS="./{data_repo_name}/cookbooks"
    DATABASES="./{data_repo_name}/generated-outputs/databases"
    DATABASES_MODULES="./{data_repo_name}/databases-modules"
    DATASETS="./{data_repo_name}/datasets"
    IO_MODULES="./{data_repo_name}/io-modules"
    METRICS="./{data_repo_name}/metrics"
    PROMPT_TEMPLATES="./{data_repo_name}/prompt-templates"
    RECIPES="./{data_repo_name}/recipes"
    RESULTS="./{data_repo_name}/generated-outputs/results"
    RESULTS_MODULES="./{data_repo_name}/results-modules"
    RUNNERS="./{data_repo_name}/generated-outputs/runners"
    RUNNERS_MODULES="./{data_repo_name}/runners-modules"
    TOKENIZERS_PARALLELISM = false
    """

    env_content_web_api = """
    # For Web API
    HOST_ADDRESS=127.0.0.1 # The interface the server will bind to
    HOST_PORT=5000

    # Below is the uri of the Web UI webhook.
    # In the next section, if Web UI listens on a different port,
    # update this uri accordingly and restart.

    MOONSHOT_UI_CALLBACK_URL=http://localhost:3000/api/v1/benchmarks/status
    """
    with open(".env", "w") as env_file:
        combined_content = env_content_data + env_content_web_api
        env_file.write(combined_content.strip())


def ms_ui_env_file(ui_repo: str) -> None:
    """
    Write the env file to be used with moonshot ui
    """
    env_content = """
    # This should be the URL of the Moonshot Web Api module which was started in the previous section.
    # Check the startup logs to determine the hostname and port number.
    MOONSHOT_API_URL=http://127.0.0.1:5000
    """
    # Write the .env content to a file
    with open(os.path.join(ui_repo, ".env"), "w") as env_file:
        env_file.write(env_content.strip())


def download_nltk_resources() -> None:
    """
    Download and verify necessary NLTK resources.

    This function downloads a predefined list of NLTK resources and verifies their availability.
    If a resource fails to download or verify, a warning is logged and an exception is raised.
    """
    import nltk

    resources = [
        "punkt",
        "stopwords",
    ]

    for resource in resources:
        try:
            nltk.download(resource)
            # Check if the resource is available
            nltk.data.find(
                f"tokenizers/{resource}"
            ) if resource == "punkt" else nltk.data.find(f"corpora/{resource}")
            logger.info(f"Successfully downloaded and verified {resource}")
        except LookupError:
            logger.warning(f"Failed to download {resource}")
            raise
        except Exception as e:
            logger.warning(f"An error occurred while downloading {resource}: {e}")
            raise


def download_spacy_model() -> None:
    """
    Downloads the en_core_web_lg model using the spacy module (for entity processor module).
    """
    subprocess.run(["python", "-m", "spacy", "download", "en_core_web_lg"])


def moonshot_data_installation(unattended: bool, overwrite: bool) -> None:
    """
    Install Moonshot Data from GitHub.

    This function clones the Moonshot Data repository from GitHub, installs its requirements,
    and sets up the necessary environment. If the target directory already exists, it handles
    the situation based on the 'unattended' and 'overwrite' flags.

    Args:
        unattended (bool): If True, the function will not prompt the user for input.
        overwrite (bool): If True, the existing directory will be removed before installation.
    """
    logger.info("Installing Moonshot Data from GitHub")
    repo = "https://github.com/aiverify-foundation/moonshot-data.git"
    folder_name = repo.split("/")[-1].replace(".git", "")
    do_install = True

    if os.path.exists(folder_name):
        if overwrite:
            logger.info(f"Removing directory {folder_name} due to --overwrite flag.")
            shutil.rmtree(folder_name)
        else:
            logger.warning(f"Directory {folder_name} already exists.")
            if not unattended:
                user_input = (
                    input(
                        f"Directory {folder_name} already exists. Do you want to remove it and reinstall? (Y/n): "
                    )
                    .strip()
                    .lower()
                )
                if user_input == "y":
                    logger.info(f"Removing directory {folder_name}.")
                    shutil.rmtree(folder_name)
                else:
                    logger.info("Skipping removal of directory.")
                    do_install = False
            else:
                do_install = False
                logger.info(
                    "Unattended mode detected. To reinstall, please use the --overwrite flag."
                )

    if do_install:
        logger.info(f"Cloning {repo}")
        # Clone the repository
        run_subprocess(["git", "clone", repo], check=True)

        # Create .env to point to installed folder
        ms_lib_env_file(folder_name)

    # Change directory to the folder
    os.chdir(folder_name)

    logger.info(f"Installing requirements for {folder_name}")
    # Install the requirements if they exist
    if os.path.exists("requirements.txt"):
        run_subprocess(["pip", "install", "-r", "requirements.txt"], check=True)
        download_nltk_resources()
        download_spacy_model()

    # Change back to the base directory
    os.chdir("..")


def check_node() -> bool:
    """
    Check if Node.js is installed on the user's machine.
    """
    try:
        result = subprocess.run(
            ["node", "--version"], capture_output=True, text=True, check=True
        )
        node_version = result.stdout.strip()
        logger.info(f"Node.js is installed. Version: {node_version}")
        return True
    except (subprocess.CalledProcessError, FileNotFoundError):
        logger.error("Node.js is not installed. Please install Node.js to proceed.")
        return False


def moonshot_ui_installation(unattended: bool, overwrite: bool) -> None:
    """
    Install Moonshot UI from GitHub.
    """
    if not check_node():
        logger.error("Node.js is not installed. Please install Node.js to proceed.")
        return

    logger.info("Installing Moonshot UI from GitHub")
    repo = "https://github.com/aiverify-foundation/moonshot-ui.git"
    folder_name = repo.split("/")[-1].replace(".git", "")
    do_install = True

    if os.path.exists(folder_name):
        if overwrite:
            logger.info(f"Removing directory {folder_name} due to --overwrite flag.")
            shutil.rmtree(folder_name)
        else:
            logger.warning(f"Directory {folder_name} already exists.")
            if not unattended:
                user_input = (
                    input(
                        f"Directory {folder_name} already exists. Do you want to remove it and reinstall? (Y/n): "
                    )
                    .strip()
                    .lower()
                )
                if user_input == "y":
                    logger.info(f"Removing directory {folder_name}.")
                    shutil.rmtree(folder_name)
                else:
                    logger.info("Skipping removal of directory.")
                    do_install = False
            else:
                do_install = False
                logger.info(
                    "Unattended mode detected. To reinstall, please use the --overwrite flag."
                )

    if do_install:
        logger.info(f"Cloning {repo}")
        # Clone the repository
        run_subprocess(["git", "clone", repo], check=True)

        # Create .env for UI
        ms_ui_env_file(folder_name)

    # Change directory to the folder
    os.chdir(folder_name)

    logger.info(f"Installing requirements for {folder_name}")
    # Install the requirements if they exist
    if os.path.exists("package.json"):
        run_subprocess(["npm", "install"], check=True)
        run_subprocess(["npm", "run", "build"], check=True)

    # Change back to the base directory
    os.chdir("..")


def run_moonshot_ui() -> None:
    """
    To start a thread to run the Moonshot UI
    """
    base_directory = os.getcwd()
    ui_dir = os.path.join(base_directory, "moonshot-ui")

    if not os.path.exists(ui_dir):
        logger.error(
            "moonshot-ui does not exist. Please run with '-i moonshot-ui' to install moonshot-ui first."
        )
        sys.exit(1)
    # ms_ui_env_file(ui_dev_dir)
    run_subprocess(["npm", "start"], cwd=ui_dir)


def main() -> None:
    parser = argparse.ArgumentParser(description="Run the Moonshot application")
    parser.add_argument(
        "mode",
        nargs="?",
        choices=["web-api", "cli", "web", "help"],
        help="Mode to run Moonshot in",
        default=help,
    )
    parser.add_argument(
        "cli_command", nargs="?", help='The CLI command to run (e.g., "interactive")'
    )
    parser.add_argument(
        "-i",
        "--install",
        action="append",
        choices=["moonshot-data", "moonshot-ui"],
        help="Modules to install",
        default=[],
    )
    parser.add_argument(
        "-e", "--env", type=str, help="Path to the .env file", default=".env"
    )
    parser.add_argument(
        "-u",
        "--unattended",
        action="store_true",
        help="Perform action without user interaction",
    )
    parser.add_argument(
        "-o",
        "--overwrite",
        action="store_true",
        help="Force delete and reinstall the specified module",
    )

    args = parser.parse_args()

    # Handle installations based on the -i include arguments
    if "moonshot-data" in args.install:
        moonshot_data_installation(args.unattended, args.overwrite)

    if "moonshot-ui" in args.install:
        moonshot_ui_installation(args.unattended, args.overwrite)

    # If mode is not specified, skip running any modes
    if args.mode is None:
        return

    if args.mode == "help":
        parser.print_help()
        sys.exit(0)

    api_set_environment_variables(dotenv_values(args.env))

    if args.mode == "web-api":
        from moonshot.integrations.web_api import __main__ as web_api

        web_api.start_app()
    elif args.mode == "web":
        # Create and start the UI dev server thread
        ui_thread = threading.Thread(target=run_moonshot_ui)
        ui_thread.start()
        ui_thread.join(timeout=0.1)  # Wait briefly for the thread to become alive
        if not ui_thread.is_alive():
            sys.exit(1)
        from moonshot.integrations.web_api import __main__ as web_api

        web_api.start_app()
    elif args.mode == "cli":
        from moonshot.integrations.cli import __main__ as cli

        cli.start_app(args.cli_command)

        # Handle CLI mode here, possibly also with additional arguments
        pass
    else:
        sys.exit(0)


if __name__ == "__main__":
    warnings.filterwarnings("ignore")
    main()



from __future__ import annotations

import textwrap
from datetime import datetime

from moonshot.src.bookmark.bookmark_arguments import BookmarkArguments
from moonshot.src.configs.env_variables import EnvVariables
from moonshot.src.storage.storage import Storage
from moonshot.src.utils.log import configure_logger

# Create a logger for this module
logger = configure_logger(__name__)


class Bookmark:
    BOOKMARK_ADD_BOOKMARK_ERROR = "[Bookmark] Failed to add bookmark record: {message}"
    BOOKMARK_ADD_BOOKMARK_SUCCESS = "[Bookmark] Bookmark added successfully."
    BOOKMARK_ADD_BOOKMARK_VALIDATION_ERROR = "Error inserting record into database."
    BOOKMARK_DELETE_ALL_BOOKMARK_ERROR = (
        "[Bookmark] Failed to delete all bookmark records: {message}"
    )
    BOOKMARK_DELETE_ALL_BOOKMARK_SUCCESS = "[Bookmark] All bookmark records deleted."
    BOOKMARK_DELETE_BOOKMARK_ERROR = (
        "[Bookmark] Failed to delete bookmark record: {message}"
    )
    BOOKMARK_DELETE_BOOKMARK_ERROR_1 = "[Bookmark] Invalid bookmark name: {message}"
    BOOKMARK_DELETE_BOOKMARK_FAIL = (
        "[Bookmark] Bookmark record not found. Unable to delete."
    )
    BOOKMARK_DELETE_BOOKMARK_SUCCESS = "[Bookmark] Bookmark record deleted."
    BOOKMARK_EXPORT_BOOKMARK_ERROR = "[Bookmark] Failed to export bookmarks: {message}"
    BOOKMARK_EXPORT_BOOKMARK_VALIDATION_ERROR = (
        "Export filename must be a non-empty string."
    )
    BOOKMARK_GET_BOOKMARK_ERROR = (
        "[Bookmark] No record found for bookmark name: {message}"
    )
    BOOKMARK_GET_BOOKMARK_ERROR_1 = "[Bookmark] Invalid bookmark name: {message}"
    _instance = None

    sql_table_name = "bookmark"

    sql_create_bookmark_table = """
        CREATE TABLE IF NOT EXISTS bookmark (
        id INTEGER PRIMARY KEY AUTOINCREMENT,
        name TEXT NOT NULL UNIQUE,
        prompt TEXT NOT NULL,
        prepared_prompt TEXT NOT NULL,
        response TEXT NOT NULL,
        context_strategy TEXT,
        prompt_template TEXT,
        attack_module TEXT,
        metric TEXT,
        bookmark_time DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP
        );
    """

    sql_insert_bookmark_record = """
        INSERT INTO bookmark (
        name, prompt, prepared_prompt, response, context_strategy, prompt_template, attack_module,
        metric, bookmark_time)
        VALUES (?,?,?,?,?,?,?,?,?);
    """

    sql_select_bookmarks_record = """
        SELECT * FROM bookmark;
    """

    sql_select_bookmark_record = """
        SELECT * FROM bookmark WHERE name = ? ;
    """

    sql_delete_bookmark_records = """
        DELETE FROM bookmark;
    """

    def __new__(cls, db_name="bookmark"):
        """
        Create a new instance of the Bookmark class or return the existing instance.

        This method ensures that only one instance of the Bookmark class is created (singleton pattern).
        If an instance already exists, it returns that instance. Otherwise, it creates a new instance
        and initializes it with the provided database name.

        Args:
            db_name (str): The name of the database. Defaults to "bookmark".

        Returns:
            Bookmark: The singleton instance of the Bookmark class.
        """
        if cls._instance is None:
            cls._instance = super(Bookmark, cls).__new__(cls)
            cls._instance.__init_instance(db_name)
        return cls._instance

    def __init_instance(self, db_name: str = "bookmark") -> None:
        """
        Initialize the database instance for the Bookmark class.

        This method sets up the database connection for the Bookmark class. It creates a new database
        connection using the provided database name and checks if the required table exists. If the table
        does not exist, it creates the table.

        Args:
            db_name (str): The name of the database. Defaults to "bookmark".
        """
        self.db_instance = Storage.create_database_connection(
            EnvVariables.BOOKMARKS.name, db_name, "db"
        )

        if not Storage.check_database_table_exists(
            self.db_instance, Bookmark.sql_table_name
        ):
            Storage.create_database_table(
                self.db_instance, Bookmark.sql_create_bookmark_table
            )

    def add_bookmark(self, bookmark: BookmarkArguments) -> dict:
        """
        Add a new bookmark to the database and return the success status.

        Args:
            bookmark (BookmarkArguments): The bookmark data to add.

        Returns:
            dict: A dictionary containing the success status and a message.
        """
        bookmark.bookmark_time = datetime.now().replace(microsecond=0).isoformat(" ")

        data = (
            bookmark.name,
            bookmark.prompt,
            bookmark.prepared_prompt,
            bookmark.response,
            bookmark.context_strategy,
            bookmark.prompt_template,
            bookmark.attack_module,
            bookmark.metric,
            bookmark.bookmark_time,
        )
        try:
            results = Storage.create_database_record(
                self.db_instance, data, Bookmark.sql_insert_bookmark_record
            )
            if results is not None:
                return {
                    "success": True,
                    "message": Bookmark.BOOKMARK_ADD_BOOKMARK_SUCCESS,
                }
            else:
                raise Exception(Bookmark.BOOKMARK_ADD_BOOKMARK_VALIDATION_ERROR)
        except Exception as e:
            return {
                "success": False,
                "message": Bookmark.BOOKMARK_ADD_BOOKMARK_ERROR.format(message=str(e)),
            }

    def get_all_bookmarks(self) -> list[dict]:
        """
        Retrieve all bookmarks from the database.

        Returns:
            list[dict]: A list of all bookmarks as dictionaries.
        """
        list_of_bookmarks_tuples = Storage.read_database_records(
            self.db_instance,
            Bookmark.sql_select_bookmarks_record,
        )
        if isinstance(list_of_bookmarks_tuples, list) and all(
            isinstance(item, tuple) for item in list_of_bookmarks_tuples
        ):
            list_of_bookmarks = [
                BookmarkArguments.from_tuple_to_dict(bookmark_tuple)
                for bookmark_tuple in list_of_bookmarks_tuples
            ]
        else:
            list_of_bookmarks = []
        return list_of_bookmarks

    def get_bookmark(self, bookmark_name: str) -> dict:
        """
        Retrieve a bookmark by its unique name.

        Args:
            bookmark_name (str): The unique name for the bookmark.

        Returns:
            dict: The bookmark information as a dictionary.

        Raises:
            RuntimeError: If the bookmark cannot be found.
        """
        if isinstance(bookmark_name, str) and bookmark_name:
            bookmark_info = Storage.read_database_record(
                self.db_instance, (bookmark_name,), Bookmark.sql_select_bookmark_record
            )
            if (
                bookmark_info is not None
                and isinstance(bookmark_info, tuple)
                and all(
                    isinstance(item, str) for item in bookmark_info[1:]
                )  # Check if the rest are strings besides id
            ):
                return BookmarkArguments.from_tuple_to_dict(bookmark_info)
            else:
                raise RuntimeError(
                    Bookmark.BOOKMARK_GET_BOOKMARK_ERROR.format(message=bookmark_name)
                )
        else:
            raise RuntimeError(
                Bookmark.BOOKMARK_GET_BOOKMARK_ERROR_1.format(message=bookmark_name)
            )

    def delete_bookmark(self, bookmark_name: str) -> dict:
        """
        Delete a bookmark by its unique name.

        Args:
            bookmark_name (str): The unique name for the bookmark to be deleted.

        Returns:
            dict: A dictionary containing the success status and a message.
        """
        if isinstance(bookmark_name, str) and bookmark_name:
            try:
                bookmark_info = Storage.read_database_record(
                    self.db_instance,
                    (bookmark_name,),
                    Bookmark.sql_select_bookmark_record,
                )
                if bookmark_info is not None:
                    sql_delete_bookmark_record = textwrap.dedent(
                        f"""
                        DELETE FROM bookmark WHERE name = '{bookmark_name}';
                    """
                    )
                    Storage.delete_database_record_in_table(
                        self.db_instance, sql_delete_bookmark_record
                    )
                    return {
                        "success": True,
                        "message": Bookmark.BOOKMARK_DELETE_BOOKMARK_SUCCESS,
                    }
                else:
                    return {
                        "success": False,
                        "message": Bookmark.BOOKMARK_DELETE_BOOKMARK_FAIL,
                    }
            except Exception as e:
                return {
                    "success": False,
                    "message": Bookmark.BOOKMARK_DELETE_BOOKMARK_ERROR.format(
                        message=str(e)
                    ),
                }
        else:
            return {
                "success": False,
                "message": Bookmark.BOOKMARK_DELETE_BOOKMARK_ERROR_1.format(
                    message=bookmark_name
                ),
            }

    def delete_all_bookmark(self) -> dict:
        """
        Delete all bookmarks from the database and return the operation result.

        Returns:
            dict: A dictionary with 'success' status and 'message' containing an error message if failed.
        """
        try:
            Storage.delete_database_record_in_table(
                self.db_instance, Bookmark.sql_delete_bookmark_records
            )
            return {
                "success": True,
                "message": Bookmark.BOOKMARK_DELETE_ALL_BOOKMARK_SUCCESS,
            }
        except Exception as e:
            return {
                "success": False,
                "message": Bookmark.BOOKMARK_DELETE_ALL_BOOKMARK_ERROR.format(
                    message=str(e)
                ),
            }

    def export_bookmarks(self, export_file_name: str = "bookmarks") -> str:
        """
        Export all bookmarks to a JSON file.

        This method retrieves all bookmarks from the database, converts them to a JSON format,
        and writes them to a file in the 'moonshot-data/bookmark' directory with the provided file name.

        Args:
            export_file_name (str): The base name of the file to export the bookmarks to.
                                    The '.json' extension will be appended to this base name.

        Returns:
            str: The path to the exported JSON file containing the bookmarks.

        Raises:
            Exception: If the export file name is invalid or an error occurs during export.
        """
        if not isinstance(export_file_name, str) or not export_file_name:
            error_message = Bookmark.BOOKMARK_EXPORT_BOOKMARK_ERROR.format(
                message=Bookmark.BOOKMARK_EXPORT_BOOKMARK_VALIDATION_ERROR
            )
            logger.error(error_message)
            raise Exception(error_message)

        list_of_bookmarks_tuples = Storage.read_database_records(
            self.db_instance,
            Bookmark.sql_select_bookmarks_record,
        )

        if (
            list_of_bookmarks_tuples is not None
            and isinstance(list_of_bookmarks_tuples, list)
            and all(isinstance(item, tuple) for item in list_of_bookmarks_tuples)
        ):
            bookmarks_json = [
                BookmarkArguments.from_tuple_to_dict(bookmark_tuple)
                for bookmark_tuple in list_of_bookmarks_tuples
            ]
        else:
            bookmarks_json = []

        try:
            return Storage.create_object(
                EnvVariables.BOOKMARKS.name,
                export_file_name,
                {"bookmarks": bookmarks_json},
                "json",
            )
        except Exception as e:
            error_message = Bookmark.BOOKMARK_EXPORT_BOOKMARK_ERROR.format(
                message=str(e)
            )
            logger.error(error_message)
            raise Exception(error_message)

    def close(self) -> None:
        """
        Close the database connection and set the Bookmark instance to None.

        This method ensures that the database connection is properly closed and the singleton
        instance of the Bookmark class is reset to None, allowing for a fresh instance to be created
        if needed in the future.
        """
        if self.db_instance:
            Storage.close_database_connection(self.db_instance)

        Bookmark._instance = None



from __future__ import annotations

from pathlib import Path

from pydantic import validate_call

from moonshot.src.configs.env_variables import EnvVariables
from moonshot.src.metrics.metric_interface import MetricInterface
from moonshot.src.storage.storage import Storage
from moonshot.src.utils.import_modules import get_instance
from moonshot.src.utils.log import configure_logger

# Create a logger for this module
logger = configure_logger(__name__)


class Metric:
    cache_name = "cache"
    cache_extension = "json"

    @classmethod
    def load(cls, met_id: str) -> Metric:
        """
        Retrieves a metric instance by its ID.

        This method attempts to load a metric instance using the provided ID. If the metric instance is found,
        it is returned. If the metric instance does not exist, a RuntimeError is raised.

        Args:
            met_id (str): The unique identifier of the metric to be retrieved.

        Returns:
            Metric: The retrieved metric instance.

        Raises:
            RuntimeError: If the metric instance does not exist.
        """
        metric_instance = get_instance(
            met_id,
            Storage.get_filepath(EnvVariables.METRICS.name, met_id, "py"),
        )
        if metric_instance:
            return metric_instance()
        else:
            raise RuntimeError(f"Unable to get defined metric instance - {met_id}")

    @staticmethod
    @validate_call
    def delete(met_id: str) -> bool:
        """
        Deletes a metric identified by its unique metric ID.

        This method attempts to delete the metric with the given ID from the storage. If the deletion is successful,
        it returns True. If an exception occurs during the deletion process, it prints an error message and re-raises
        the exception.

        Args:
            met_id (str): The unique identifier of the metric to be deleted.

        Returns:
            bool: True if the metric was successfully deleted.

        Raises:
            Exception: If an error occurs during the deletion process.
        """
        try:
            Storage.delete_object(EnvVariables.METRICS.name, met_id, "py")
            return True

        except Exception as e:
            logger.error(f"Failed to delete metric: {str(e)}")
            raise e

    @staticmethod
    def get_cache_information() -> dict:
        """
        Retrieves cache information from the storage.

        This method attempts to read the cache information from the storage and return it as a dictionary.
        If the cache information does not exist or an error occurs, it returns an empty dictionary.

        Returns:
            dict: A dictionary containing the cache information or an empty dictionary if an error occurs
            or if the cache information does not exist.

        Raises:
            Exception: If there's an error during the retrieval process, it is logged and an
            empty dictionary is returned.
        """
        try:
            # Retrieve cache information from the storage and return it as a dictionary
            cache_info = Storage.read_object(
                EnvVariables.METRICS.name, Metric.cache_name, Metric.cache_extension
            )
            return cache_info if cache_info else {}
        except Exception:
            logger.error(
                f"No previous cache information because {Metric.cache_name} is not found."
            )
            return {}

    @staticmethod
    def write_cache_information(cache_info: dict) -> None:
        """
        Writes the updated cache information to the storage.

        Args:
            cache_info (dict): The cache information to be written.
        """
        try:
            Storage.create_object(
                obj_type=EnvVariables.METRICS.name,
                obj_id=Metric.cache_name,
                obj_info=cache_info,
                obj_extension=Metric.cache_extension,
            )
        except Exception as e:
            logger.error(f"Failed to write cache information: {str(e)}")
            raise e

    @staticmethod
    def get_available_items() -> tuple[list[str], list[dict]]:
        """
        Retrieves a list of available metric names and their corresponding information.

        This method scans the storage for metric objects, filters out any system or cache-related entries,
        and compiles a list of metric names and their detailed information. It also checks the cache for
        any updates and writes back to the cache if necessary.

        Returns:
            tuple[list[str], list[dict]]: A tuple containing two lists, one with the names of the metrics
                                           and the other with the corresponding metric information dictionaries.
        """
        try:
            retn_mets = []
            retn_mets_ids = []
            met_cache_info = Metric.get_cache_information()
            cache_needs_update = False  # Initialize a flag to track cache updates
            mets = Storage.get_objects(EnvVariables.METRICS.name, "py")

            for met in mets:
                if "__" in met or MetricInterface.config_name in met:
                    continue

                met_name = Path(met).stem
                met_info, cache_updated = Metric._get_or_update_metrics_info(
                    met_name, met_cache_info
                )
                if cache_updated:
                    cache_needs_update = True  # Set the flag if any cache was updated

                retn_mets.append(met_info)
                retn_mets_ids.append(met_name)

            if cache_needs_update:  # Check the flag after the loop
                Metric.write_cache_information(met_cache_info)

            return retn_mets_ids, retn_mets

        except Exception as e:
            logger.error(f"Failed to get available metrics: {str(e)}")
            raise e

    @staticmethod
    def _get_or_update_metrics_info(
        met_name: str, met_cache_info: dict
    ) -> tuple[dict, bool]:
        """
        Retrieves or updates the metric information from the cache.

        This method checks if the metric information is already available in the cache and if the file hash matches
        the one stored in the cache. If it does, the information is retrieved from the cache. If not, the metric
        information is read from the storage, the cache is updated with the new information and the new file hash,
        and a flag is set to indicate that the cache has been updated.

        Args:
            met_name (str): The name of the metric.
            met_cache_info (dict): A dictionary containing the cached metric information.

        Returns:
            tuple[dict, bool]: A tuple containing the dictionary with the metric information
                               and a boolean indicating whether the cache was updated or not.
        """
        file_hash = Storage.get_file_hash(EnvVariables.METRICS.name, met_name, "py")
        cache_updated = False

        if met_name in met_cache_info and file_hash == met_cache_info[met_name]["hash"]:
            met_metadata = met_cache_info[met_name].copy()
            met_metadata.pop("hash", None)
        else:
            met_metadata = Metric.load(met_name).get_metadata()  # type: ignore ; ducktyping
            met_cache_info[met_name] = met_metadata.copy()
            met_cache_info[met_name]["hash"] = file_hash
            cache_updated = True

        return met_metadata, cache_updated



from pydantic import BaseModel, Field


class CookbookArguments(BaseModel):
    id: str  # id (str): The unique identifier for the Cookbook.

    name: str = Field(min_length=1)  # name (str): The name of the Cookbook.

    description: str  # description (str): A brief description of the Cookbook.

    tags: list[str]  # tags (list): The list of tags in the Cookbook.

    categories: list[str]  # categories (list): The list of categories in the Cookbook.

    recipes: list[str] = Field(
        min_length=1
    )  # recipes (list): A list of recipes included in the Cookbook.

    def to_dict(self) -> dict:
        """
        Converts the CookbookArguments instance into a dictionary.

        This method takes all the attributes of the CookbookArguments instance and constructs a dictionary
        with attribute names as keys and their corresponding values. This includes the id, name, description,
        and recipes.

        This dictionary can be used for serialization purposes, such as storing the cookbook information in a JSON file
        or sending it over a network.

        Returns:
            dict: A dictionary representation of the CookbookArguments instance.
        """
        return {
            "id": self.id,
            "name": self.name,
            "tags": self.tags,
            "categories": self.categories,
            "description": self.description,
            "recipes": self.recipes,
        }



def display_view_list_format(title: str, items: list) -> str:
    """
    Format a list of items for display.

    This function takes a title and a list of items and formats them into a string suitable for display.
    Each item in the list is displayed on a new line with an index number. If the list is empty, it returns
    the title with 'nil'.

    Args:
        title (str): The title to display above the list.
        items (list): A list of items to be formatted.

    Returns:
        str: The formatted list as a string.
    """
    if items:
        return f"[blue]{title}[/blue]:" + "".join(
            f"\n{i + 1}. {item}" for i, item in enumerate(items)
        )
    else:
        return f"[blue]{title}[/blue]: nil"


def display_view_str_format(title: str, item: str) -> str:
    """
    Format a string item for display with a title.

    This function takes a title and a string item and formats them into a string suitable for display.
    If the item is not an empty string, it is displayed next to the title with a blue color for the title.
    If the item is an empty string, 'nil' is displayed next to the title.

    Args:
        title (str): The title to display next to the item.
        item (str): The string item to be formatted.

    Returns:
        str: The formatted string with the title and item.
    """
    if item:
        return f"[blue]{title}[/blue]: {item}"
    else:
        return f"[blue]{title}[/blue]: nil"



from moonshot.src.utils.find_feature import find_keyword
from moonshot.src.utils.pagination import get_paginated_lists


def filter_data(
    list_of_data: list, keyword: str = "", pagination: tuple = ()
) -> list | None:
    """
    Filters and paginates a list of data.

    Args:
        list_of_data (list): The list of data to be filtered and paginated.
        keyword (str, optional): The keyword to filter the data. Defaults to "".
        pagination (tuple, optional): A tuple containing the page number and page size. Defaults to ().

    Returns:
        list | None: The filtered and paginated list of data, or None if no data matches the criteria.
    """
    # if there is a find keyword
    if keyword:
        list_of_data = find_keyword(keyword, list_of_data)
        if not list_of_data:
            return

    # if pagination is required
    if pagination:
        # add index to every dictionary in the list
        if all(isinstance(item, dict) for item in list_of_data):
            for index, item in enumerate(list_of_data, 1):
                if isinstance(item, dict):
                    item["idx"] = index

        # get paginated data
        page_number = pagination[0]
        page_size = pagination[1]

        if page_number <= 0 or page_size <= 0:
            # print("Invalid page number or page size. Page number and page size should start from 1.")
            raise RuntimeError(
                "Invalid page number or page size. Page number and page size should start from 1."
            )

        paginated_data = get_paginated_lists(page_size, list_of_data)

        # perform index checks
        paginated_data_size = len(paginated_data)
        if page_number > paginated_data_size:
            list_of_data = paginated_data[(paginated_data_size - 1)]
        else:
            list_of_data = paginated_data[(page_number - 1)]

    return list_of_data



from ast import literal_eval

import cmd2
from rich.console import Console
from rich.table import Table

from moonshot.api import (
    api_convert_dataset,
    api_delete_dataset,
    api_download_dataset,
    api_get_all_datasets,
    api_get_all_datasets_name,
)
from moonshot.integrations.cli.cli_errors import (
    ERROR_BENCHMARK_DELETE_DATASET_DATASET_VALIDATION,
    ERROR_BENCHMARK_LIST_DATASETS_FIND_VALIDATION,
    ERROR_BENCHMARK_LIST_DATASETS_PAGINATION_VALIDATION,
    ERROR_BENCHMARK_LIST_DATASETS_PAGINATION_VALIDATION_1,
    ERROR_BENCHMARK_VIEW_DATASET_DATASET_FILENAME_VALIDATION,
)
from moonshot.integrations.cli.common.display_helper import display_view_str_format
from moonshot.integrations.cli.utils.process_data import filter_data

console = Console()


def list_datasets(args) -> list | None:
    """
    List all available datasets.

    This function retrieves all available datasets by calling the api_get_all_datasets function from the
    moonshot.api module. It then filters the datasets based on the provided keyword and pagination arguments.
    If there are no datasets, it prints a message indicating that no datasets were found.

    Args:
        args: A namespace object from argparse. It should have optional attributes:
            find (str): Optional keyword to filter datasets.
            pagination (str): Optional tuple to paginate datasets.

    Returns:
        list | None: A list of datasets or None if there are no datasets.
    """
    try:
        print("Listing datasets may take a while...")
        if args.find is not None:
            if not isinstance(args.find, str) or not args.find:
                raise TypeError(ERROR_BENCHMARK_LIST_DATASETS_FIND_VALIDATION)

        if args.pagination is not None:
            if not isinstance(args.pagination, str) or not args.pagination:
                raise TypeError(ERROR_BENCHMARK_LIST_DATASETS_PAGINATION_VALIDATION)
            try:
                pagination = literal_eval(args.pagination)
                if not (
                    isinstance(pagination, tuple)
                    and len(pagination) == 2
                    and all(isinstance(i, int) for i in pagination)
                ):
                    raise ValueError(
                        ERROR_BENCHMARK_LIST_DATASETS_PAGINATION_VALIDATION_1
                    )
            except (ValueError, SyntaxError):
                raise ValueError(ERROR_BENCHMARK_LIST_DATASETS_PAGINATION_VALIDATION_1)
        else:
            pagination = ()

        datasets_list = api_get_all_datasets()
        keyword = args.find.lower() if args.find else ""

        if datasets_list:
            filtered_datasets_list = filter_data(datasets_list, keyword, pagination)
            if filtered_datasets_list:
                _display_datasets(filtered_datasets_list)
                return filtered_datasets_list

        console.print("[red]There are no datasets found.[/red]")
        return None

    except Exception as e:
        print(f"[list_datasets]: {str(e)}")
        return None


def view_dataset(args) -> None:
    """
    View a specific dataset.

    This function retrieves all available datasets and their names by calling the api_get_all_datasets and
    api_get_all_datasets_name functions. It then finds the dataset with the name specified in args.dataset_filename
    and displays it using the _display_datasets function. If an exception occurs, it prints an error message.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            dataset_filename (str): The name of the dataset to view.

    Returns:
        None
    """
    try:
        print("Viewing datasets may take a while...")
        if (
            not isinstance(args.dataset_filename, str)
            or not args.dataset_filename
            or args.dataset_filename is None
        ):
            raise TypeError(ERROR_BENCHMARK_VIEW_DATASET_DATASET_FILENAME_VALIDATION)

        datasets_list = api_get_all_datasets()
        datasets_name_list = api_get_all_datasets_name()

        # Find the index of the dataset with the name args.dataset_filename
        dataset_index = datasets_name_list.index(args.dataset_filename)
        # Pass the corresponding dataset from datasets_list to _display_datasets
        _display_datasets([datasets_list[dataset_index]])

    except Exception as e:
        print(f"[view_dataset]: {str(e)}")


def delete_dataset(args) -> None:
    """
    Delete a dataset.

    This function deletes a dataset with the specified name. It prompts the user for confirmation before proceeding
    with the deletion. If the user confirms, it calls the api_delete_dataset function from the moonshot.api module to
    delete the dataset. If the deletion is successful, it prints a confirmation message. If an exception occurs, it
    prints an error message.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            dataset (str): The name of the dataset to delete.

    Returns:
        None
    """
    # Confirm with the user before deleting a dataset
    confirmation = console.input(
        "[bold red]Are you sure you want to delete the dataset (y/N)? [/]"
    )
    if confirmation.lower() != "y":
        console.print("[bold yellow]Dataset deletion cancelled.[/]")
        return

    try:
        if (
            args.dataset is None
            or not isinstance(args.dataset, str)
            or not args.dataset
        ):
            raise ValueError(ERROR_BENCHMARK_DELETE_DATASET_DATASET_VALIDATION)

        api_delete_dataset(args.dataset)
        print("[delete_dataset]: Dataset deleted.")
    except Exception as e:
        print(f"[delete_dataset]: {str(e)}")


def convert_dataset(args) -> None:
    """
    Convert an existing dataset to a new format.

    Args:
        args: A namespace object from argparse with the following attributes:
            - name (str): Name of the new dataset.
            - description (str): Description of the new dataset.
            - reference (str): Reference of the new dataset.
            - license (str): License of the new dataset.
            - csv_file_path (str): Path to the existing dataset file.

    Returns:
        None
    """
    try:
        new_dataset_id = api_convert_dataset(
            args.name,
            args.description,
            args.reference,
            args.license,
            args.csv_file_path,
        )
        print(f"[convert_dataset]: Dataset ({new_dataset_id}) created.")
    except Exception as e:
        print(f"[convert_dataset]: {str(e)}")


def download_dataset(args) -> None:
    """
    Download a dataset from Hugging Face.

    Args:
        args: A namespace object from argparse with the following attributes:
            - name (str): Name of the new dataset.
            - description (str): Description of the new dataset.
            - reference (str): Reference of the new dataset.
            - license (str): License of the new dataset.
            - params (dict): Parameters for the dataset in dictionary format.

    Returns:
        None
    """
    try:
        new_dataset_id = api_download_dataset(
            args.name,
            args.description,
            args.reference,
            args.license,
            **args.params,
        )
        print(f"[download_dataset]: Dataset ({new_dataset_id}) created.")
    except Exception as e:
        print(f"[download_dataset]: {str(e)}")


# ------------------------------------------------------------------------------
# Helper functions: Display on cli
# ------------------------------------------------------------------------------
def _display_datasets(datasets_list: list):
    """
    Displays a list of datasets in a table format.

    This function takes a list of datasets and displays them in a table format with each dataset's name, description,
    and other relevant details. If the list is empty, it prints a message indicating that no datasets are found.

    Args:
        datasets_list (list): A list of dictionaries, where each dictionary contains the details of a dataset.

    Returns:
        None
    """
    table = Table(
        title="List of Datasets", show_lines=True, expand=True, header_style="bold"
    )
    table.add_column("No.", width=2)
    table.add_column("Dataset", justify="left", width=78)
    for idx, dataset in enumerate(datasets_list, 1):
        (
            id,
            name,
            description,
            _,
            num_of_dataset_prompts,
            created_date,
            reference,
            license,
            *other_args,
        ) = dataset.values()

        idx = dataset.get("idx", idx)
        prompt_info = display_view_str_format("Prompts", num_of_dataset_prompts)
        created_date_info = display_view_str_format("Created Date", created_date)
        license_info = display_view_str_format("License", license)
        reference_info = display_view_str_format("Reference", reference)

        dataset_info = (
            f"[red]{id}[/red]\n\n[blue]{name}[/blue]\n{description}\n\n"
            f"{prompt_info}\n\n{created_date_info}\n\n{license_info}\n\n{reference_info}"
        )

        table.add_section()
        table.add_row(str(idx), dataset_info)
    console.print(table)


# ------------------------------------------------------------------------------
# Cmd2 Arguments Parsers
# ------------------------------------------------------------------------------
# View dataset arguments
view_dataset_args = cmd2.Cmd2ArgumentParser(
    description="View a dataset file.",
    epilog="Example:\n view_dataset bbq-lite-age-ambiguous",
)
view_dataset_args.add_argument(
    "dataset_filename", type=str, help="Name of the dataset file"
)

# Delete dataset arguments
delete_dataset_args = cmd2.Cmd2ArgumentParser(
    description="Delete a dataset.",
    epilog="Example:\n delete_dataset bbq-lite-age-ambiguous",
)
delete_dataset_args.add_argument("dataset", type=str, help="Name of the dataset")

# List dataset arguments
list_datasets_args = cmd2.Cmd2ArgumentParser(
    description="List all datasets.",
    epilog='Example:\n list_datasets -f "bbq"',
)

list_datasets_args.add_argument(
    "-f",
    "--find",
    type=str,
    help="Optional field to find dataset(s) with keyword",
    nargs="?",
)

list_datasets_args.add_argument(
    "-p",
    "--pagination",
    type=str,
    help="Optional tuple to paginate dataset(s). E.g. (2,10) returns 2nd page with 10 items in each page.",
    nargs="?",
)

# Convert dataset arguments
convert_dataset_args = cmd2.Cmd2ArgumentParser(
    description="Convert your dataset. The 'name' argument will be slugified to create a unique identifier.",
    epilog=(
        "Examples:\n"
        "convert_dataset 'dataset-name' 'A brief description' 'http://reference.com' 'MIT' '/path/to/your/file.csv'"
    ),
)
convert_dataset_args.add_argument("name", type=str, help="Name of the new dataset")
convert_dataset_args.add_argument(
    "description", type=str, help="Description of the new dataset"
)
convert_dataset_args.add_argument(
    "reference", type=str, help="Reference of the new dataset"
)
convert_dataset_args.add_argument(
    "license", type=str, help="License of the new dataset"
)
convert_dataset_args.add_argument(
    "csv_file_path", type=str, help="Path to your existing dataset"
)


# Download dataset arguments
download_dataset_args = cmd2.Cmd2ArgumentParser(
    description="Download dataset from Hugging Face. The 'name' argument will be slugified to create a unique ID.",
    epilog=(
        "Examples:\n"
        "download_dataset 'dataset-name' 'A brief description' 'http://reference.com' 'MIT' "
        "\"{'dataset_name': 'cais/mmlu', 'dataset_config': 'college_biology', 'split': 'dev', "
        "'input_col': ['question','choices'], 'target_col': 'answer'}\""
    ),
)
download_dataset_args.add_argument("name", type=str, help="Name of the new dataset")
download_dataset_args.add_argument(
    "description", type=str, help="Description of the new dataset"
)
download_dataset_args.add_argument(
    "reference", type=str, help="Reference of the new dataset"
)
download_dataset_args.add_argument(
    "license", type=str, help="License of the new dataset"
)
download_dataset_args.add_argument(
    "params",
    type=literal_eval,
    help=(
        "Params of the new dataset in dictionary format. For example: \n"
        "{'dataset_name': 'cais_mmlu', 'dataset_config': 'college_biology', 'split': 'test', "
        "'input_col': ['questions','choices'], 'target_col': 'answer'}\""
    ),
)



from abc import abstractmethod

from moonshot.src.utils.timeit import timeit


class ContextStrategyInterface:
    @abstractmethod
    @timeit
    def get_metadata(self) -> dict | None:
        """
        Abstract method to retrieve metadata from the context strategy.

        Returns:
            dict | None: Returns a dictionary containing the metadata of the context strategy, or None if the
            operation was unsuccessful.
        """
        pass

    @abstractmethod
    def add_in_context(
        self, user_prompt: str, list_of_previous_prompts: list[dict] = []
    ) -> None:
        """
        Abstract method to add a user prompt and list of previous prompts to the context strategy.

        Args:
            user_prompt (str): The user prompt to be added to the context strategy.
            list_of_previous_prompts (list[dict], optional): List of previous prompts. Defaults to [].

        Returns:
            None
        """
        pass

    @abstractmethod
    def get_number_of_prev_prompts(self, no_prev_prompts: int) -> int:
        """
        Abstract method to get the number of previous prompts to be retrieved from the context strategy.

        Args:
            no_prev_prompts (int): The number of previous prompts to retrieve.

        Returns:
            int: The number of previous prompts to be retrieved.
        """
        pass



import logging
import os

import uvicorn
from dotenv import dotenv_values, load_dotenv

from .app import create_app
from .container import Container
from .logging_conf import configure_app_logging, create_uvicorn_log_config
from .services.utils.file_manager import create_temp_dir
from .types.types import UvicornRunArgs


def start_app():
    load_dotenv()
    container: Container = Container()
    # use our own config.yml
    config_file = dotenv_values().get("MS_WEB_API_CONFIG")
    if config_file is None:
        container.config.from_default()
    else:
        container.config.from_yaml(f"{config_file}", required=True)

    create_temp_dir(container.config.temp_folder())
    configure_app_logging(container.config)
    logging.info(f"Environment: {container.config.app_environment()}")
    ENABLE_SSL = container.config.ssl.enabled()
    SSL_CERT_PATH = container.config.ssl.file_path()
    app = create_app(container.config)

    run_kwargs: UvicornRunArgs = {}
    port = dotenv_values().get("HOST_PORT", 5000)
    if port is not None:
        port = int(port)
    run_kwargs["port"] = port
    run_kwargs["host"] = dotenv_values().get("HOST_ADDRESS", "127.0.0.1")
    run_kwargs["log_config"] = create_uvicorn_log_config(container.config)
    if ENABLE_SSL:
        if not SSL_CERT_PATH:
            logging.debug("SSL_CERT_PATH not set, not enabling SSL")
        elif os.path.exists(os.path.join(SSL_CERT_PATH, "key.pem")) and os.path.exists(
            os.path.join(SSL_CERT_PATH, "cert.pem")
        ):
            run_kwargs["ssl_keyfile"] = str(
                os.path.join(SSL_CERT_PATH, str(container.config.ssl.key_filename()))
            )
            run_kwargs["ssl_certfile"] = str(
                os.path.join(SSL_CERT_PATH, str(container.config.ssl.cert_filename()))
            )
        else:
            logging.debug(
                "SSL_CERT_PATH does not contain necessary files, not enabling SSL"
            )
    uvicorn.run(app, **run_kwargs)  # type: ignore


if __name__ == "__main__":
    start_app()



import logging
import os
from pathlib import Path


def configure_logger(name: str):
    """
    Configures and returns a logger with a specified name.

    This function creates a logger and sets its level to INFO. It also creates a console handler,
    sets its level to INFO, and assigns a specific formatter to it. The formatter includes the
    timestamp, log level, filename, function name, line number, and the log message. Finally, the
    console handler is added to the logger.

    Args:
        name (str): The name of the logger to be created and configured.

    Returns:
        logging.Logger: The configured logger with the specified name.
    """
    log_extension = ".log"

    # Read environment variable
    log_filename = os.getenv("MS_LOG_NAME", "moonshot").lower()
    log_level = os.getenv("MS_LOG_LEVEL", "INFO").upper()
    log_write_to_file = os.getenv("MS_LOG_TO_FILE", "false").lower() == "true"

    # Create a formatter
    formatter = logging.Formatter(
        "%(asctime)s [%(levelname)s][%(filename)s::%(funcName)s(%(lineno)d)] %(message)s"
    )

    # Check name is valid
    if not name or not isinstance(name, str) or name is None:
        name = Path(__file__).stem

    # Check log level is valid
    valid_log_levels = ["DEBUG", "INFO", "WARNING", "ERROR"]
    if log_level not in valid_log_levels:
        log_level = "INFO"

    # Check log filename is valid
    if not log_filename or not isinstance(log_filename, str) or log_filename is None:
        log_filename = "moonshot"

    logger = logging.getLogger(name)
    logger.setLevel(log_level)
    logger.propagate = False

    # Check if there is already a stream handler
    if not any(
        isinstance(handler, logging.StreamHandler) for handler in logger.handlers
    ):
        # Create a console handler
        console_handler = logging.StreamHandler()
        console_handler.setLevel(log_level)
        console_handler.setFormatter(formatter)
        logger.addHandler(console_handler)

    # Create a file handler if required
    if log_write_to_file and not any(
        isinstance(handler, logging.FileHandler) for handler in logger.handlers
    ):
        file_path = Path(".").joinpath(log_filename).with_suffix(log_extension)
        file_handler = logging.FileHandler(
            str(file_path)
        )  # Convert Path object to string
        file_handler.setLevel(log_level)
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)

    return logger



from pydantic import validate_call

from moonshot.src.prompt_templates.prompt_template import PromptTemplate


# ------------------------------------------------------------------------------
# Prompt Template APIs
# ------------------------------------------------------------------------------
def api_get_all_prompt_template_detail() -> list[dict]:
    """
    Retrieves all available prompt template details and returns them as a list of dictionaries.

    This function calls the `get_all_prompt_template_details` method from the `PromptTemplate` class
    to fetch the details of all prompt templates. It then returns these details as a list of dictionaries.

    Returns:
        list[dict]: A list of dictionaries, each representing the details of a prompt template.
    """
    return PromptTemplate.get_all_prompt_template_details()


def api_get_all_prompt_template_name() -> list[str]:
    """
    Retrieves all available prompt template names and returns them as a list.

    This function calls the `get_all_prompt_template_names` method from the `PromptTemplate` class
    to fetch the names of all prompt templates. It then returns these names as a list of strings.

    Returns:
        list[str]: A list of prompt template names.
    """
    return PromptTemplate.get_all_prompt_template_names()


@validate_call
def api_delete_prompt_template(pt_id: str) -> bool:
    """
    Deletes a prompt template by its identifier.

    This function calls the `delete` method from the `PromptTemplate` class to delete a prompt template
    identified by its unique ID. It returns True if the deletion was successful, otherwise it raises an exception.

    Args:
        pt_id (str): The unique identifier of the prompt template to be deleted.

    Returns:
        bool: True if the prompt template was successfully deleted.

    Raises:
        Exception: If the deletion process encounters an error.
    """
    return PromptTemplate.delete(pt_id)



from typing import Any, Union


def find_keyword(
    keyword: str, target: Union[str, list[str], list[list[str]], list[dict], list, dict]
) -> Any:
    """
    Find the keyword in the target and return the matching elements.

    Args:
        keyword (str): The keyword to search for.
        target: The target to search within.

    Returns:
        Any: The matching elements or None if no match is found.
    """

    # find in single string
    if isinstance(target, str):
        if target.find(keyword) != -1:
            return target
        return ""

    # find in single dict
    elif isinstance(target, dict):
        if any(str(keyword) in str(value).lower() for value in target.values()):
            return target
        return {}

    # find in list
    elif isinstance(target, list):
        # list of string
        if all(isinstance(item, str) for item in target):
            return [item for item in target if str(item).lower().find(keyword) != -1]

        # list of dict
        elif all(isinstance(item, dict) for item in target):
            return [
                item
                for item in target
                if isinstance(item, dict)
                and any(str(keyword) in str(value).lower() for value in item.values())
            ]

    return None



from .... import api as moonshot_api
from ..services.base_service import BaseService
from ..services.utils.exceptions_handler import exception_handler


class MetricService(BaseService):
    @exception_handler
    def get_all_metric(self) -> list[dict]:
        metrics = moonshot_api.api_get_all_metric()
        return metrics

    @exception_handler
    def delete_metric(self, metric_id: str) -> None:
        moonshot_api.api_delete_metric(metric_id)



from .... import api as moonshot_api
from .base_service import BaseService
from .utils.exceptions_handler import exception_handler


class AttackModuleService(BaseService):
    @exception_handler
    def get_all_attack_module(self) -> list[str]:
        """
        Retrieves a list of all available attack modules.

        This method wraps the `api_get_all_attack_modules` function from the moonshot API and returns
        a list of attack module names.

        Returns:
            list[str]: A list of strings, each denoting the name of an attack module.
        """
        attack_modules = moonshot_api.api_get_all_attack_modules()
        return attack_modules

    @exception_handler
    def get_all_attack_module_metadata(self) -> list[dict]:
        """
        Retrieves metadata for all available attack modules.

        This method calls the `api_get_all_attack_module_metadata` function from the moonshot API. It collects
        metadata for each attack module, which includes details such as the module's name, description, and
        any other relevant information provided by the module's `get_metadata` method.

        Returns:
            list[dict]: A list of dictionaries, each containing metadata of an attack module.
        """
        am_metadata = moonshot_api.api_get_all_attack_module_metadata()
        return am_metadata



import time
from functools import wraps

from moonshot.src.utils.log import configure_logger

# Create a logger for this module
logger = configure_logger(__name__)


def timeit(func):
    @wraps(func)
    def timeit_wrapper(*args, **kwargs):
        """
        A decorator function that measures the execution time of the decorated function.

        Args:
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Returns:
            The result of the decorated function.
        """
        start_time = time.perf_counter()
        result = func(*args, **kwargs)
        end_time = time.perf_counter()
        total_time = end_time - start_time
        logger.debug(
            f"[{func.__module__}] Running [{func.__name__}] took {total_time:.4f}s"
        )
        return result

    return timeit_wrapper
